#include "maxis.h"

/////////////
// Voronoi //
/////////////

vector<Point> Voronoi::getVertex(){
    return this->vertex;
}

vector< pair<Point, Point> > Voronoi::getEdges(){
    vector< pair<Point, Point> > segments;
    float x,y;

    for (int i=0; i<this->edges.size(); i++){
        pair<Point, Point> p;

        if (this->edges[i].v1 == -1){
            if (-0.001 < this->coeff[this->edges[i].line].b 
            and this->coeff[this->edges[i].line].b < 0.001){
                 x = this->coeff[this->edges[i].line].c;
                 y = 100000;
            }
            else{
                x = -100000;
                y = -(this->coeff[this->edges[i].line].a * x + this->coeff[this->edges[i].line].c) /    this->coeff[this->edges[i].line].b;
            
            }
            p.first = Point(x, y);
        }
        else{
            p.first = this->vertex[this->edges[i].v1];
        }
        
        if (this->edges[i].v2 == -1){
            if (-0.001 < this->coeff[this->edges[i].line].b 
            and this->coeff[this->edges[i].line].b < 0.001){
                 x = this->coeff[this->edges[i].line].c;
                 y = -100000;
            }
            else{
                x = 100000;
                y = -(this->coeff[this->edges[i].line].a * x + this->coeff[this->edges[i].line].c) /    this->coeff[this->edges[i].line].b;
            
            }
            p.second = Point(x, y);
        }
        else{
            p.second = this->vertex[this->edges[i].v2];
        }        
        segments.push_back(p);
    }
    return segments;
}


void Voronoi::_executeFortunes(){
    system("../../lib/aux/maxis/voronoi < ../../lib/aux/maxis/inputfile.txt > ../../lib/aux/maxis/outputfile.txt");
}


void Voronoi::_readOutputFile(){
    float x, y;
    Coefficient c;
    int l, v1, v2;
    
    char line[100];
    int i;

    FILE *file;
    file = fopen("../../lib/aux/maxis/outputfile.txt", "r");

    while(!feof(file))
    {
        fgets(line, 100, file);
        stringstream line_elements;
        line_elements.str(line);
        string type;
       
        line_elements >> type;
        if (type=="v"){
            line_elements >> x >> y;
            vertex.push_back(Point(x, y));
            
        }
        if (type=="l"){
            line_elements >> c.a >> c.b >> c.c;
            coeff.push_back(c);
        }
        if (type=="e"){
            line_elements >> l >> v1 >> v2;
            MaxisEdge e;
            e.line = l;
            e.v1 = v1;
            e.v2 = v2;
            edges.push_back(e);
        }
    }
    fclose(file);

}


void Voronoi::_writePoints(){
    FILE *file;
    file = fopen("../../lib/aux/maxis/inputfile.txt", "w");

    for (unsigned int i = 0; i < vertex.size(); i++){
        fprintf(file, "%f %f\n", vertex[i].getX(), vertex[i].getZ());
    }
    fclose(file);
}

void Voronoi::_smooth(){
	double kernel[] = { 0.0625, 0.25, 0.375, 0.25, 0.0625 };

	for ( int l = 0; l < 200; l++ ) {
		vector<Point> new_points( vertex.size() );
		for ( size_t j=0; j<vertex.size() && vertex.size() > 4; j++){
			Point a(0,0);
			for ( int k=-2; k<3; k++) {
				int c = ( vertex.size() + j + k)%vertex.size();
				a+= vertex[c] * kernel[k + 2];
			}
			new_points[j] = a;
		}
		vertex.clear();
		vertex = new_points;
	}
}

void Voronoi::setPoints(vector<Point> points){
    vertex = points;
    _smooth();
    _writePoints();
    vertex.clear();
}

void Voronoi::createVoronoi(){
    this->_executeFortunes();
    this->_readOutputFile();
    for (int i = 0; i < vertex.size(); i++){
        vertex[i].setZ(vertex[i].getY());
        vertex[i].setY(0.0);
    }
}


void Voronoi::clear(){
    this->vertex.clear();
    this->coeff.clear();
    this->edges.clear();
}
