#ifndef GRAPHIC_OBJECT_H
#define GRAPHIC_OBJECT_H

#include <GL/gl.h>

#include "primitive.h"

#define POLYGON 1
#define LINE    2
#define POINTS  3

class GraphicObject{

    protected:
        unsigned int id;
        
        int type;
        
        float r,g,b,a;
        int size;
        
        float rotate_x;
        float rotate_y;
        float rotate_z;

        float translate_x;
        float translate_y;
        float translate_z;

        float scale_x;
        float scale_y;
        float scale_z;        
        
    public:
         GraphicObject(int _id = 0);
         ~GraphicObject();

        void setId(int _id);
        int getId();
        
        void setColor(float _r, float _g, float _b, float _a = 1.0);
        float getRed();
        float getGreen();
        float getBlue();
        float getAlpha();
        void setSize(int s);
        int getSize();
        
        void setRotateX(float rx);
        float getRotateX();
        void setRotateY(float ry);
        float getRotateY();
        void setRotateZ(float rz);
        float getRotateZ();
        void rotate();
        void rotate(float angle, Point direction);
        
        void setTranslation(float tx, float ty, float tz);
        void setTranslateX(float tx);
        float getTranslateX();
        void setTranslateY(float ty);
        float getTranslateY();
        void setTranslateZ(float tz);
        float getTranslateZ();
        void translate();
        
        void setScaleX(float sx);
        float setScaleX();
        void setScaleY(float sy);
        float setScaleY();
        void setScaleZ(float sz);
        float setScaleZ();
        void scale();
        
        void mode(int mode);
        
        virtual void draw(){};
};

#endif // GRAPHIC_OBJECT_H
