#include "primitive.h"

#include <cstdio>
#include <cmath>

#include <GL/gl.h>
#include <GL/glut.h>

/////////////
//  Point  //
/////////////

Point::Point(){            
    this->x = 0.0;
    this->y = 0.0;
    this->z = 0.0;

    this->r = -1.0;
    this->g = -1.0;
    this->b = -1.0;
    this->s = -1;
}

Point::Point(float _x, float _y, float _z){
    this->x = _x;
    this->y = _y;
    this->z = _z;

    this->r = -1.0;
    this->g = -1.0;
    this->b = -1.0;
    this->s = -1;
}

Point::~Point(){

}

void Point::setAtributtes(float _r, float _g, float _b, int _s){
    this->r = _r;
    this->g = _g;
    this->b = _b;
    this->s = _s;
}


void Point::setX(float x){
    this->x = x;
}

float Point::getX(){
    return this->x;
}

void Point::setY(float y){
    this->y = y;
}

float Point::getY(){
    return this->y;
}

void Point::setZ(float z){
    this->z = z;
}

float Point::getZ(){
    return this->z;
}        

void Point::show(){
    printf("(%f,   %f,   %f)\n", x, y, z);
}   

float Point::lenght(){
    return sqrt( pow(x, 2) + pow(y, 2) + pow(z, 2));
}

float Point::distance(Point p){
    return sqrt( pow(p.getX() - this->x, 2) + 
                 pow(p.getY() - this->y, 2) + 
                 pow(p.getZ() - this->z, 2) );
}

bool Point::operator==(Point p){
    if (distance(p) > -0.1 and distance(p) < 0.00001){
        return true;
    }
    else{
        return false;
    }
}

void Point::operator+=(Point p){
    this->x += p.getX();
    this->y += p.getY();
    this->z += p.getZ();
}

Point Point::operator+(Point p){
    Point r;
    r.setX( this->x + p.getX() );
    r.setY( this->y + p.getY() );
    r.setZ( this->z + p.getZ() );
    return r;
}

void Point::operator-=(Point p){
    this->x -= p.getX();
    this->y -= p.getY();
    this->z -= p.getZ();
}
        
Point Point::operator-(Point p){
    Point r;
    r.setX( this->x - p.getX() );
    r.setY( this->y - p.getY() );
    r.setZ( this->z - p.getZ() );
    return r;
}        

Point Point::operator-(){
    return *this * (-1);
}

void Point::operator*=(float h){
    this->x *= h;
    this->y *= h;
    this->z *= h;
}

Point Point::operator*(float h){
    Point r;
    r.setX( this->x * h );
    r.setY( this->y * h );
    r.setZ( this->z * h );
    return r;
}

void Point::operator/=(float h){
    this->x /= h;
    this->y /= h;
    this->z /= h;
}

Point Point::operator/(float h){
    Point r;
    r.setX( this->x / h );
    r.setY( this->y / h );
    r.setZ( this->z / h );
    return r;
}

void Point::translate(Point t){
    this->operator+=(t);
}

void Point::rotate(int axis, float angle){
    float _x = x;
    float _y = y;
    float _z = z;
    if (axis == 0){
        y = (_y*cos(angle))-(_z*sin(angle));
        z = (_y*sin(angle))+(_z*cos(angle));
    }
    else if (axis == 1){
        x = (_x*cos(angle))+(_z*sin(angle));
        z = -(_x*sin(angle))+(_z*cos(angle));
    }
    else if (axis == 2){
        x = (_x*cos(angle))-(_y*sin(angle));
        y = (_x*sin(angle))+(_y*cos(angle));
    }
}

void Point::rotate(Point p){
    rotate(0, p.getX());
    rotate(1, p.getY());
    rotate(2, p.getZ());
}

void Point::rotate(float ax, float ay, float az){
    rotate(0, ax);
    rotate(1, ay);
    rotate(2, az);
}

void Point::scale(float s){
    this->operator*=(s);
}

Point Point::crossProduct(Point p){
    return Point( this->y * p.getZ() - this->z * p.getY(),
                  this->z * p.getX() - this->x * p.getZ(),
                  this->x * p.getY() - this->y * p.getX() );
}


float Point::dotProduct(Point p){
    return this->x * p.getX() + this->y * p.getY() + this->z * p.getZ();
}


float Point::angle(Point p){
    Point q = normalize();
    p = p.normalize();
    return acos(p.dotProduct(q));
}

Point Point::normalize(){
    return this->operator/( lenght() );
}


void Point::draw(float _r, float _g, float _b, int _s){
    if (_r >= 0 and _g >= 0 and _b >= 0 ){
        glColor3f(_r, _g, _b);
    }
    else if (this->r >= 0 and this->g >= 0 and this->b >= 0 ){
        glColor3f(this->r, this->g, this->b);
    }
    if (_s > 0 and _s < 10){
        glPointSize(_s);
    }
    else if (this->s > 0 and this->s < 10){
        glPointSize(this->s);
    }
    glVertex3f(x, y, z);
}


//////////////
//  Sphere  //
//////////////

Sphere::Sphere(Point _center, float _radius, int _id){
    this->center = _center;
    this->radius = _radius;
    this->id = _id;
}

void Sphere::draw(){
//    if (id >=0) glLoadName(id);
    glPushMatrix();
        glTranslatef(center.getX(), center.getY(), center.getZ());
        GLUquadricObj *pObj;
        pObj = gluNewQuadric();
        gluQuadricNormals(pObj, GLU_SMOOTH);
        gluSphere(pObj, radius, 25, 15);
        gluDeleteQuadric(pObj);
    glPopMatrix();        
}


////////////////
//  Cylinder  //
////////////////

Cylinder::Cylinder(Point _center, float _radius, Point _generatrix){
    this->center = _center;
    this->radius = _radius;
    this->generatrix = _generatrix;
}

void Cylinder::rotate(){
    Point up(0.0, 0.0, generatrix.lenght());
    Point normal = generatrix.crossProduct(up);
                
    float angle = acos( generatrix.dotProduct(up) 
                        / ( generatrix.lenght() * up.lenght() ) );
    
    glRotatef(-angle/3.14159265*180, normal.getX(), normal.getY(), normal.getZ());
}

void Cylinder::draw(){
    glPushMatrix();
        glTranslatef(center.getX(), center.getY(), center.getZ());
        rotate();
        GLUquadricObj *pObj;
        pObj = gluNewQuadric();
        gluQuadricNormals(pObj, GLU_SMOOTH);
        gluCylinder(pObj, radius, radius, generatrix.lenght(), 25, 15);
        gluDeleteQuadric(pObj);
    glPopMatrix();        
}


////////////
//  Cone  //
////////////

Cone::Cone(Point _center, float _radius, Point _generatrix){
    this->center = _center;
    this->radius = _radius;
    this->generatrix = _generatrix;
}

void Cone::rotate(){
    Point up(0.0, 0.0, generatrix.lenght());
    Point normal = generatrix.crossProduct(up);
                
    float angle = acos( generatrix.dotProduct(up) 
                        / ( generatrix.lenght() * up.lenght() ) );
    
    glRotatef(-angle/3.14159265*180, normal.getX(), normal.getY(), normal.getZ());
}

void Cone::draw(){
    glPushMatrix();
        glTranslatef(center.getX(), center.getY(), center.getZ());
        rotate();
//        glRotatef(90.0,1.0,0.0,0.0);
        GLUquadricObj *pObj;
        pObj = gluNewQuadric();
        gluQuadricNormals(pObj, GLU_SMOOTH);
        gluCylinder(pObj, 0.0, radius, generatrix.lenght(), 25, 25);
        gluDeleteQuadric(pObj);
    glPopMatrix();        
}


///////////////
//  Polygon  //
///////////////

Polygon::Polygon(){}
Polygon::Polygon(vector<Point> _vertices){
    this->vertices = _vertices;
}

void Polygon::addVertex(Point v){
    vertices.push_back(v);
}

void Polygon::draw(float r, float g, float b){
    if (vertices.size() > 0){
        GLUtesselator *pTess = gluNewTess();
        
        glColor4f(r, g, b, 1.0); 

        gluTessCallback(pTess, GLU_TESS_VERTEX, (GLvoid(*)()) &glVertex3dv);
        gluTessCallback(pTess, GLU_TESS_BEGIN, (GLvoid(*)()) &glBegin);
        gluTessCallback(pTess, GLU_TESS_END, (GLvoid(*)()) &glEnd);
        
        gluTessBeginPolygon(pTess, NULL);
        gluTessBeginContour(pTess);

        GLdouble p[this->vertices.size()][3];
        for(int i = 0; i < this->vertices.size(); i++){
            p[i][0] = vertices[i].getX();
            p[i][1] = vertices[i].getY();
            p[i][2] = vertices[i].getZ();
            gluTessVertex(pTess, p[i], p[i]); 
        }
            
        gluTessEndContour(pTess);
        gluTessEndPolygon(pTess);
        
        gluDeleteTess(pTess);
    }
}


/////////////
//  Plane  //
/////////////

Plane::Plane(){
    normal = Point(0.0, 1.0, 0.0);
    center = Point(0.0, 0.0, 0.0);
}

Plane::Plane(Point _normal, Point _center){
    normal = _normal;
    center = _center;
}

vector<Point> Plane::calculatePoints(){
    vector<Point> points;
    
    if (normal.getZ() < -0.1 or normal.getZ() > 0.1 ){
        points.push_back( Point( -200, -200, 
                                 - normal.getX() * (-200) / normal.getZ()
                                 - normal.getY() * (-200) / normal.getZ()) );

        points.push_back( Point( +200, -200, 
                                 - normal.getX() * (+20) / normal.getZ()
                                 - normal.getY() * (-20) / normal.getZ()) );
                                 
        points.push_back( Point( +200, +200, 
                                 - normal.getX() * (+200) / normal.getZ()
                                 - normal.getY() * (+200) / normal.getZ()) );
                                 
        points.push_back( Point( -200, +200, 
                                 - normal.getX() * (-200) / normal.getZ()
                                 - normal.getY() * (+200) / normal.getZ()) );
    }
    else if (normal.getY() < -0.1 or normal.getY() > 0.1 ){
        points.push_back( Point( -200, -200, 
                                 - normal.getX() * (-200) / normal.getY()
                                 - normal.getZ() * (-200) / normal.getY()) );

        points.push_back( Point( +200, -200, 
                                 - normal.getX() * (+200) / normal.getY()
                                 - normal.getZ() * (-200) / normal.getY()) );
                                 
        points.push_back( Point( +200, +200, 
                                 - normal.getX() * (+200) / normal.getY()
                                 - normal.getZ() * (+200) / normal.getY()) );
                                 
        points.push_back( Point( -200, +200, 
                                 - normal.getX() * (-200) / normal.getY()
                                 - normal.getZ() * (+200) / normal.getY()) );
    }
    else if (normal.getX() < -0.1 or normal.getX() > 0.1 ){
        points.push_back( Point( -200, -200, 
                                 - normal.getY() * (-200) / normal.getX()
                                 - normal.getZ() * (-200) / normal.getX()) );

        points.push_back( Point( +200, -200, 
                                 - normal.getY() * (+200) / normal.getX()
                                 - normal.getZ() * (-200) / normal.getX()) );
                                 
        points.push_back( Point( +200, +200, 
                                 - normal.getY() * (+200) / normal.getX()
                                 - normal.getZ() * (+200) / normal.getX()) );
                                 
        points.push_back( Point( -200, +200, 
                                 - normal.getY() * (-200) / normal.getX()
                                 - normal.getZ() * (+200) / normal.getX()) );
    }
    for (int i = 0; i < points.size(); i++){
        points[i] += center;
    }
    return points;        
}        

void Plane::setCenter(Point _c){
    this->center = _c;
}

void Plane::setNormal(Point _n){
    this->normal = _n;
}

void Plane::draw(float r, float g, float b){
    vector<Point> points = calculatePoints();
    glColor3f(r, g, b); 
    glBegin(GL_POLYGON);
        for (int i=0; i<points.size(); i++){
            glVertex3f( points[i].getX(), points[i].getY(), points[i].getZ() );
        }
    glEnd();
}


///////////
//  Box  //
///////////

Point origin;
float width;
float height;
float depth;

Box::Box(){
    this->width  = -1;
    this->height = -1;
    this->depth  = -1;
}

Box::~Box(){

}

bool isEmpty(){
    if (origin.lenght() < 0.1 and width < 0.0 and height < 0.0 and depth < 0.0){
        return true;
    }
    return false;
}

void Box::resizeBox(vector<Point> points){
    if (points.size() > 2){
        if ( isEmpty() ){
            this->origin = points[0];
            this->width  = abs( points[0].getX() - points[1].getX() );
            this->height = abs( points[0].getY() - points[1].getY() );
            this->depth  = abs( points[0].getZ() - points[1].getZ() );    
        }

        for (int i = 0; i < points.size(); i++){
            if (this->origin.getX() > points[i].getX() ){
                this->origin.setX( points[i].getX() - 0.2 );
            }
            if (this->origin.getY() > points[i].getY() ){
                this->origin.setY( points[i].getY() - 0.2 );
            }
            if (this->origin.getZ() > points[i].getZ() ){
                this->origin.setZ( points[i].getZ() - 0.2 );
            }
            if ( this->width < abs( this->origin.getX() - points[i].getX() ) ){
                this->width = abs( this->origin.getX() - points[i].getX() ) + 0.2;
            }
            if ( this->height < abs( this->origin.getY() - points[i].getY() ) ){
                this->height = abs( this->origin.getY() - points[i].getY() ) + 0.2;
            }
            if ( this->depth < abs( this->origin.getZ() - points[i].getZ() ) ){
                this->depth = abs( this->origin.getZ() - points[i].getZ() ) + 0.2;
            }
        }
    }
}

void Box::draw(float r, float g, float b){
    glColor3f(r, g, b); 
    glBegin(GL_LINE_STRIP);
        origin.draw();
        (origin + Point(width, 0.0, 0.0)).draw();
        (origin + Point(width, height, 0.0)).draw();
        (origin + Point(0.0, height, 0.0)).draw();
        origin.draw();
    glEnd();
    glBegin(GL_LINE_STRIP);
        (origin + Point(0.0, 0.0, depth)).draw();
        (origin + Point(width, 0.0, depth)).draw();
        (origin + Point(width, height, depth)).draw();
        (origin + Point(0.0, height, depth)).draw();
        (origin + Point(0.0, 0.0, depth)).draw();
    glEnd();
    glBegin(GL_LINE_STRIP);
        origin.draw();
        (origin + Point(0.0, 0.0, depth)).draw();
        (origin + Point(0.0, height, depth)).draw();
        (origin + Point(0.0, height, 0.0)).draw();
        origin.draw();
    glEnd();
    glBegin(GL_LINE_STRIP);
        (origin + Point(width, 0.0, 0.0)).draw();
        (origin + Point(width, 0.0, depth)).draw();
        (origin + Point(width, height, depth)).draw();
        (origin + Point(width, height, 0.0)).draw();
        (origin + Point(width, 0.0, 0.0)).draw();
    glEnd();
}

