#ifndef PRIMITIVE_H
#define PRIMITIVE_H

#include <vector>
using namespace std;

class Point{
    protected:
        float x;
        float y;
        float z;
        
        float r, g, b;
        int s;
        
        int id;

    public:
        Point();
        Point(float _x, float _y, float _z = 0.0);
        ~Point();

        void setId(int _id){
            this->id = _id;
        }
        
        int getId(){
            return this->id;
        }
        
        void setAtributtes(float _r, float _g, float _b, int _s = 1);
            
        void setX(float x);
        float getX();
        
        void setY(float y);
        float getY();
        
        void setZ(float z);
        float getZ();       

        void show();
        
        float lenght();
        
        float distance(Point p);
        
        bool operator==(Point p);
        
        void operator+=(Point p);
        Point operator+(Point p);
        
        void operator-=(Point p);
        Point operator-(Point p);
        Point operator-();
        
        void operator*=(float h);
        Point operator*(float h);
        
        void operator/=(float h);
        Point operator/(float h);
        
        void translate(Point t);
        void rotate(int axis, float angle);
        void rotate(Point p);
        void rotate(float ax, float ay, float az);
        void scale(float s);
        
        Point crossProduct(Point p);
        float dotProduct(Point p);
        float angle(Point p);
        Point normalize();
        
        Point projection(Point p){
            return this->operator*(dotProduct(p) / lenght() );
        }
        
        void draw(float r=-1.0, float g=-1.0, float b=-1.0, int s=-1);
};


class Sphere{
    private:
        float radius;
        Point center;
        int id;
        
    public:
        Sphere(Point _center, float _radius, int _id = -1);
        void draw();
};


class Cylinder{
    private:
        float radius;
        Point center;
        Point generatrix;
        
    public:
        Cylinder(Point _center, float _radius, Point _generatrix);
        
        void rotate();
        void draw();
};


class Cone{
    private:
        float radius;
        Point center;
        Point generatrix;
        
    public:
        Cone(Point _center, float _radius, Point _generatrix);
        
        void rotate();
        void draw();
};


class Polygon{
    private:
        vector<Point> vertices;
        
    public:
        Polygon();
        Polygon(vector<Point> _vertices);
        
        void addVertex(Point v);
        void draw(float r, float g, float b);
};

class Plane{
    private:
        Point normal;
        Point center;
        
        vector<Point> calculatePoints();
        
    public:
        Plane();
        Plane(Point _normal, Point _center);
        
        void setCenter(Point _c);
        void setNormal(Point _n);

        void draw(float r, float g, float b);
};

class Box{
    public:
        Point origin;
        float width;
        float height;
        float depth;
    
        Box();
        ~Box();
        
        void resizeBox(vector<Point> points);
        void draw(float r, float g, float b);
};
#endif // PRIMITIVE_H
