#include "surface.h"

#include <cmath>

Pvert::Pvert() : Vertex(), position(0,0,0), color(0.0, 0.0, 0.0){

}

Pvert::~Pvert(){

}

Pvert* Pvert::cast(Vertex* v) {
    return static_cast<Pvert *>(v);
}

Point Pvert::getNormal(){
    return normal;
}

Point Pvert::setNormal(Point n){
    normal = n;
}

void Pvert::chooseColor(){
    srand ( time(NULL) );
    float h = position.getY();
    if (h < 0.05)
        color = Point (0, 0, 150 + (rand() % 10)*10);
    else if (h + sin(position.getX())*cos(position.getY()) < 2.0)
         color = Point(175  + rand() % 40, 145 + rand() % 40, 101  + rand() % 40);
    else if (h - sin(position.getY())*cos(position.getX()) < 5.0)
        color = Point(124, 252, 0);
    else if (h + sin(position.getZ())*cos(position.getY()) < 8.0)
        color = Point(85, 107, 47);
    else
        color = Point (250, 250, 250);

    color.show();

}

Point Pvert::getColor(){
    return color;
}

void Pvert::setColor(Point c){
    color = c;
}

Dem::Dem(Point origin, float w, float d) {
    double mpts[] = { 
        origin.getX(),     origin.getY(), origin.getZ(),
        origin.getX(),     origin.getY(), origin.getZ() + d,
        origin.getX() + w, origin.getY(), origin.getZ() + d,
        origin.getX() + w, origin.getY(), origin.getZ(),
    };
    int mfcs[] = {
      0, 2, 3,
      2, 0, 1,
    };
    basic = false;
    bumpmapping = false;

    target_level = 1;
    ythresh = 0.0; 
    band = 1.01;

    np = 4;
    nf = 2;
    pts = new double[np*3];
    fcs = new int[nf*3];

    for (int i = 0; i < np*3; i++)
        pts[i] = mpts[i];
    for (int i = 0; i < nf*3; i++)
        fcs[i] = mfcs[i];
}  

Dem::Dem() {
    double mpts[] = { 
      0, 0, 0,
      1, 0, 0,
      0, 1, 0,
      1, 1, 0,
    };
    int mfcs[] = {
      0, 2, 3,
      2, 0, 1,
    };
    basic = false;
    bumpmapping = false;

    target_level = 1;
    ythresh = 0.0; 

    np = 4;
    nf = 2;
    pts = new double[np*3];
    fcs = new int[nf*3];

    for (int i = 0; i < np*3; i++)
      pts[i] = mpts[i];
    for (int i = 0; i < nf*3; i++)
      fcs[i] = mfcs[i];
}

Dem::~Dem() { 
    delete[] pts; 
    delete[] fcs; 
}

float Dem::getHeight(Point p){
    if (dem.size() > 0){
        int rows = dem.size();
        int cols = dem[0].size();
        float widthInc = width/float(rows);
        float depthInc = depth/float(cols);

        int r = ( (width / 2.0)  + p.getX() ) / widthInc;
        int c = ( (depth / 2.0) + p.getZ() ) / depthInc;

        float tx = ( ( (width / 2.0) + p.getX() ) - (widthInc*float(r)) ) / widthInc;
        float tz = ( ( (depth / 2.0) + p.getZ() ) - (depthInc*float(c)) ) / depthInc;

        float d00 = dem[(r<rows-1?r:rows-1)][(c<cols-1?c:cols-1)];
        float d01 = dem[(r<rows-2?(r+1):r-1)][(c<cols-1?c:cols-1)];
        float d10 = dem[(r<rows-1?r:rows-1)][(c<cols-2?(c+1):c-1)];
        float d11 = dem[(r<rows-2?(r+1):r-1)][(c<cols-2?(c+1):c-1)];
        
        float hx0 = (1 - tx)*d00 + tx*d01;
        float hx1 = (1 - tx)*d10 + tx*d11;
        
        return (1-tz)*hx0 + tz*hx1;
    }
    return 0.0;
}

Point Dem::getColor(Point p){
    if (colors.size() > 0){
        int rows = colors.size();
        int cols = colors[0].size();
        float widthInc = width/float(rows);
        float depthInc = depth/float(cols);

        int r = ( (width / 2.0)  + p.getX() ) / widthInc;
        int c = ( (depth / 2.0) + p.getZ() ) / depthInc;
        return colors[(r<rows-1?r:rows-1)][(c<cols-1?c:cols-1)];
    }
    return Point(0.0, 0.0, 0.0);
}

Point Dem::getNormal(Point p){
        Point n, s, e, w;

    if (dem.size() > 0){
        int rows = dem.size();
        int cols = dem[0].size();
        float widthInc = width/float(rows);
        float depthInc = depth/float(cols);

        int r = ( (width / 2.0)  + p.getX() ) / widthInc;
        r = r < rows-1 ? r : rows-1;
        int c = ( (depth / 2.0) + p.getZ() ) / depthInc;
        c = c < cols-1 ? c : cols-1;

        float tx = ( ( (width / 2.0) + p.getX() ) - (widthInc*float(r)) ) / widthInc;
        float tz = ( ( (depth / 2.0) + p.getZ() ) - (depthInc*float(c)) ) / depthInc;

        Point pos(widthInc*float(r), dem[r][c], depthInc*float(c));

        if (r>0 and c<cols-1)
            w = (pos - Point(widthInc*float(r-1), dem[r-1][c+1], depthInc*float(c+1))).normalize();
        if (r>0 and c>0)
            n = (pos - Point(widthInc*float(r-1), dem[r-1][c-1], depthInc*float(c-1))).normalize();
        if (r<rows-1 and c>0)
            e = (pos - Point(widthInc*float(r+1), dem[r+1][c-1], depthInc*float(c-1))).normalize();
        if (r<rows-1 and c<cols-1)
            s = (pos - Point(widthInc*float(r+1), dem[r+1][c+1], depthInc*float(c+1))).normalize();

        Point n0 = n.crossProduct(w) * (1 - tx) + w.crossProduct(s) * tx;
        Point n1 = e.crossProduct(n) * (1 - tx) + s.crossProduct(e) * tx;


        Point n = ( n0 * (1-tz) + n1 * tz ).normalize();
        if (bumpmapping){
            return n + bumpNoise[r][c];
        }
        return n;
    }
}


void Dem::setDem(vector< vector<float> > _dem){
    dem.clear();
    dem = _dem;

    bumpNoise.clear();
    bumpNoise.resize(dem.size());
    for(int i = 0; i < bumpNoise.size(); i++){
        bumpNoise[i].resize(dem[0].size());
        for(int j = 0; j < bumpNoise[0].size(); j++){
            bumpNoise[i][j] = Point(float(rand()%30)/100.0, float(rand()%30)/100.0, float(rand()%30)/100.0);
        }
    }
}

void Dem::setDimensions(int w, int d){
    width = w;
    depth = d;
}

void Dem::setContour(Sketch c){
    contour = c;
}

void Dem::setCamera(Point cam){
    camera = cam;
}

Point Dem::getCamera(){
    return camera;
}

void Dem::setBasic(bool b){
    basic = b;
}

void Dem::setBumpMapping(bool bump){
    bumpmapping = bump;

}

bool Dem::getBumpMapping(){
    return bumpmapping;
}

void Dem::base_mesh(int *pnp, int **pfcs, int *pnf){
    *pnp = np;
    *pfcs = fcs;
    *pnf = nf;
}

void Dem::sample(int k, Vertex* v){
    int i = k * 3; 
    Pvert* pv = Pvert::cast(v);
    pv->position = Point(pts[i], pts[i+1], pts[i+2]);
    pv->position.setY(getHeight(pv->position));


    pv->setColor(getColor(pv->position));

    pts[i+1] = pv->position.getY();

    if (pv->getNormal().lenght() < 0.1) pv->setNormal(getNormal(pv->position));
}

void Dem::sample(Edge* e, Vertex* v){
    Pvert* v0 = Pvert::cast(e->org());
    Pvert* v1 = Pvert::cast(e->dst());

    Pvert *pv = Pvert::cast(v);
    pv->position = (v0->position + v1->position) * 0.5;
    pv->position.setY(getHeight(pv->position));

    pv->setColor(getColor(pv->position));

    if (pv->getNormal().lenght() < 0.1) pv->setNormal(getNormal(pv->position));

}

void Dem::sample(Face *f, Vertex* v){
    Pvert* v0 = Pvert::cast(f->vertex(0));
    Pvert* v1 = Pvert::cast(f->vertex(1));
    Pvert* v2 = Pvert::cast(f->vertex(2));

    Pvert *pv = Pvert::cast(v);
    pv->position = (v0->position + v1->position + v2->position) / 3.0;
    pv->position.setY(getHeight(pv->position));

    pv->setColor(getColor(pv->position));

    if (pv->getNormal().lenght() < 0.1) pv->setNormal(getNormal(pv->position));
}

float Dem::elenght(Edge* e){
    Pvert* v0 = Pvert::cast(e->org());
    Pvert* v1 = Pvert::cast(e->dst());
    return (v1->position - v0->position).lenght();
}


//////////////////////////
//   Angle adaptation   //
//////////////////////////

//float Dem::ref_rank(Edge *e){
//    Pvert* v0 = Pvert::cast(e->org());
//    Pvert* v1 = Pvert::cast(e->dst());
//
//    float angle = (v0->a - camera).angle(v1->a - camera) * 180.0 / 3.14159265;
//
//    if (false and angle > 0.8){
//        printf("\nRef Angle: %f \t\t",angle);
//        v0->a.show();
//        v1->a.show();
//        (v0->a - camera).show();
//        (v1->a - camera).show();
//    }
//
//    if (!basic){
//        if (angle > 1.0)
//            return 1;
//        else
//            return -1;
//    }
//    else{
//        return (e->level() <= 1)? 1 : -1;
//    }
//}
//
//
//
//float Dem::simpl_rank(Vertex *w){
//    Pvert* v0;
//    Pvert* v1;
//    Pvert* v = Pvert::cast(w);
//
//    Hedge* edge = w->star_first();
//    edge = w->star_first();
//    v0 = Pvert::cast(edge->org());
//    edge = w->star_next(edge);
//    edge = w->star_next(edge);
//    v1 = Pvert::cast(edge->org());
//
//    float angle = (v0->a - camera).angle(v1->a - camera) * 180.0 / 3.14159265;
//
//    if (false and angle < 1.0){
//        printf("\nSimp Angle: %f \t\t",angle);
//        v0->a.show();
//        v1->a.show();
//        (v0->a - camera).show();
//        (v1->a - camera).show();
//    }
//
//    if (!basic){
//        if (angle < 1.0){
//            return 1;
//        }
//        else{
//            return -1;
//        }
//    }
//    else{
//        return (v->level() <= 1)? 1 : -1;
//    }
//}


////////////////////////////
//   Default adaptation   //
////////////////////////////

//float Dem::ref_rank(Edge *e){
//    Pvert* v0 = Pvert::cast(e->org());
//    Pvert* v1 = Pvert::cast(e->dst());
//    return (e->level() < target_level-1)? 1 : -1;
//}
//
//float Dem::simpl_rank(Vertex *w){
//    Pvert* v = Pvert::cast(w);
//    return (v->level() > target_level)? 1 : -1;
//}



///////////////////////////////
////   Distance adaptation   //
///////////////////////////////

float Dem::ref_rank(Edge *e){
    Pvert* v0 = Pvert::cast(e->org());
    Pvert* v1 = Pvert::cast(e->dst());
    Point p = (v0->position + v1->position) * 0.5;

    if (!basic){
        if (p.distance(camera) > 80.0) return (e->level() < target_level-4)? 1 : -1;
        if (p.distance(camera) > 75.0) return (e->level() < target_level-3)? 1 : -1;
        if (p.distance(camera) > 70.0) return (e->level() < target_level-2)? 1 : -1;
        if (p.distance(camera) > 65.0) return (e->level() < target_level-1)? 1 : -1;
        if (p.distance(camera) > 60.0) return (e->level() < target_level)? 1 : -1;
        if (p.distance(camera) > 55.0) return (e->level() < target_level+1)? 1 : -1;
        if (p.distance(camera) > 50.0) return (e->level() < target_level+2)? 1 : -1;
        if (p.distance(camera) > 45.0) return (e->level() < target_level+3)? 1 : -1;
        if (p.distance(camera) > 40.0) return (e->level() < target_level+4)? 1 : -1;
        if (p.distance(camera) > 35.0) return (e->level() < target_level+5)? 1 : -1;
        return -1;
    }
    else{
        return (e->level() <= 1)? 1 : -1;
    }
}

float Dem::simpl_rank(Vertex *w){
    Pvert* v = Pvert::cast(w);
    if (!basic){
        if (v->position.distance(camera) > 80.0) return (v->level() > target_level-4)? 1 : -1;
        if (v->position.distance(camera) > 75.0) return (v->level() > target_level-3)? 1 : -1;
        if (v->position.distance(camera) > 70.0) return (v->level() > target_level-2)? 1 : -1;
        if (v->position.distance(camera) > 65.0) return (v->level() > target_level-1)? 1 : -1;
        if (v->position.distance(camera) > 60.0) return (v->level() > target_level)? 1 : -1;
        if (v->position.distance(camera) > 55.0) return (v->level() > target_level+1)? 1 : -1;
        if (v->position.distance(camera) > 50.0) return (v->level() > target_level+2)? 1 : -1;
        if (v->position.distance(camera) > 45.0) return (v->level() > target_level+3)? 1 : -1;
        if (v->position.distance(camera) > 40.0) return (v->level() > target_level+4)? 1 : -1;
        if (v->position.distance(camera) > 35.0) return (v->level() > target_level+5)? 1 : -1;
        return -1;
    }
    else{
       return (v->level() >= 1)? 1 : -1;
    }
}

Vertex* Dem::new_vertex(void){
    return new Pvert(); 
}

void Dem::del_vertex(Vertex *v){
    delete(Pvert::cast(v)); 
}

void Dem::change_level(int i){
    target_level += i;
    if (target_level <= 0){
        target_level = 1;
    }
//    printf("Level: %i (t_l)\n",target_level);
    band = 100.0 - target_level*4.0;
}
