#ifndef SURFACE_H
#define SURFACE_H

#include <stdlib.h>
#include <time.h>

#include "a48.h"
#include <vector>
#include <GL/glut.h> 

#include "primitive.h"
#include "sketch.h"

using namespace A48;

class Pvert : public Vertex {
    private:
        Point normal;
        Point color;

    public:
        Point position;

        Pvert();
        ~Pvert();

        static Pvert* cast(Vertex* v);
        Point getNormal();
        Point setNormal(Point n);

        void chooseColor();
        Point getColor();
        void setColor(Point c);
};

class Dem : public virtual Surface {
    public:
        double* pts;
        int* fcs;
        int np, nf;
        int target_level;
        double ythresh; 
        double band;
        Point camera;
        
        float width;
        float depth;

        bool basic;
        bool bumpmapping;

        vector< vector<float> > dem;
        vector< vector<Point> > colors;
        vector< vector<Point> > bumpNoise;
        Sketch contour;

        Dem(Point origin, float w, float d);
        Dem();
        ~Dem();
        
        float getHeight(Point p);
        Point getNormal(Point p);
        Point getColor(Point p);

        void setDem(vector< vector<float> > _dem);
        void setColors(vector< vector<Point> > c){colors = c;}
        void setDimensions(int w, int d);

        void setContour(Sketch c);
        void setCamera(Point cam);
        Point getCamera();

        void setBasic(bool b);

        void setBumpMapping(bool bump);
        bool getBumpMapping();

        void base_mesh(int *pnp, int **pfcs, int *pnf);

        void sample(int k, Vertex* v);
        void sample(Edge* e, Vertex* v);
        void sample(Face *f, Vertex* v);

        float elenght(Edge* e);

        float ref_rank(Edge *e);
        float simpl_rank(Vertex *w);

        Vertex* new_vertex(void);
        void del_vertex(Vertex *v);

        void change_level(int i);

};

#endif // SURFACE_H
