#include "camera.h"

#include <cstdio>

#include <GL/gl.h>
#include <GL/glu.h>

////////////////
//  Viewport  //
////////////////

Viewport::Viewport(){
    this->left = 0;
    this->top = 0;
    this->width = 1;
    this->height = 1;
}

Viewport::Viewport(int _left, int _top, int _width, int _height){
    this->setViewport(_left, _top, _width, _height);
}

Viewport::~Viewport(){

}

void Viewport::setViewport(int _left, int _top, int _width, int _height){
    this->left = _left;
    this->top = _top;
    this->width = _width;
    this->height = _height;
}

//////////////
//  Window  //
//////////////

Window::Window(){
    this->left = 0;
    this->right = 0;
    this->bottom = 1;
    this->top = 1;
}

Window::Window(float l, float r, float b, float t, float n, float f){
    this->setWindow(l, r, b, t, n, f);
}

Window::~Window(){

}

void Window::setWindow(float l, float r, float b, float t, float n, float f){
    this->left = l;
    this->right = r;
    this->bottom = b;
    this->top = t;
    this->near = n;
    this->far = f;
}

//////////////
//  Camera  //
//////////////

Camera::Camera(){
    position = Point(0.0, 1.0, 0.0);
    lookat = Point(0.0, 0.0, 0.0);
    up = Point(0.0, 0.0, -1.0);
    
    this->aspect = -1.0;
    this->fov = -1.0;
    this->perspective = false;
}

Camera::~Camera(){

}

void Camera::setPosition(float x, float y, float z){
    position.setX(x);
    position.setY(y);
    position.setZ(z);
}

void Camera::setLookat(float x, float y, float z){
    lookat.setX(x);
    lookat.setY(y);
    lookat.setZ(z);
}

void Camera::setUp(float x, float y, float z){
    up.setX(x);
    up.setY(y);
    up.setZ(z);
}

void Camera::setViewport(int _left, int _top, int _width, int _height){
    this->viewport.setViewport(_left, _top, _width, _height);
}

Viewport& Camera::getViewport(){
    return this->viewport;
}

void Camera::setWindow(float l, float r, float b, float t, float n, float f){
    this->window.setWindow(l, r, b, t, n, f);
}

Window& Camera::getWindow(){
    return this->window;
}

void Camera::setFov(float f){
    this->fov = f;
}

void Camera::setAspectRatio(float ar){
    this->aspect = ar;
}

void Camera::enablePerspective(){
    this->perspective = true;
}

void Camera::disablePerspective(){
    this->perspective = false;
}

void Camera::resize(int w, int h){
    if (w == 0) w = 1;
    if (h == 0) h = 1;
    this->viewport.width = w;
    this->viewport.height = h;
    
    if (this->aspect < 0) this->aspect = (float)w / (float)h;
    if (this->fov < 0) this->fov = 15.0;

    if (perspective){
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
    }

    glOrtho( window.left, window.right, window.bottom, window.top, 
             window.near, window.far);
    
    if (perspective){
        gluPerspective(this->fov , this->aspect, window.near, window.far);
    
        gluLookAt( position.getX(), position.getY(), position.getZ(),
                   lookat.getX(),   lookat.getY(),   lookat.getZ(),
                   up.getX(),       up.getY(),       up.getZ() );

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
    }
}

void Camera::show(){
    printf("\n\nFOV: %f\n",fov);
    printf("aspect: %f\n",aspect);
    printf("Viewport w: %i  h: %i\n", viewport.width, viewport.height);
    printf("Window l: %f  r: %f  b: %f  t: %f  n: %f  f: %f\n",
            window.left, window.right, window.bottom, window.top, window.near, window.far);
            
    printf("Position: ");position.show();
    printf("Lookat: ");lookat.show();
    printf("Up: ");up.show();
}
