#ifndef CAMERA_H
#define CAMERA_H

#include "primitive.h"

class Viewport{
    public:
        int left;
        int top;
        int width;
        int height;
        
        Viewport();
        Viewport(int _left, int _top, int _width, int _height);
        ~Viewport();

        void setViewport(int _left, int _top, int _width, int _height);
};

class Window{
    public:
        float left;
        float right;
        float bottom;
        float top;
        float near;
        float far;
        
        Window();
        Window(float l, float r, float b, float t, float n, float f);
        ~Window();

        void setWindow(float l, float r, float b, float t, float n=-10, float f=10);
};

class Camera{
    public:
        Point position;
        Point lookat;
        Point up;
        
        Viewport viewport;
        Window window;
        
        float fov;
        float aspect;
        
        bool perspective;
        
        Camera();
        ~Camera();
        
        void setPosition(float x, float y, float z);
        void setLookat(float x, float y, float z);
        void setUp(float x, float y, float z);
        
        void setViewport(int _left, int _top, int _width, int _height);
        Viewport &getViewport();
        
        void setWindow(float l, float r, float b, float t, float n, float f);
        Window &getWindow();
        
        void setFov(float f);
        
        void setAspectRatio(float f);
        
        void enablePerspective();
        void disablePerspective();
        
        void resize(int w, int h);
        
        void show();
};

#endif // CAMERA_H
