#include "scene.h"

#include <cstdio>
#include <GL/gl.h>
#include <GL/glu.h>

#include "canvas.h"
#include "camera.h"

////////////////
//  Selected  //
////////////////

Selected::Selected(){

}

Selected::~Selected(){

}

void Selected::show(){
    printf("%i   %i   %i   %i\n", count, zmin, zmax, id);
}

/////////////
//  Scene  //
/////////////

Scene::Scene(Canvas* c){
    canvas = c;
}

Scene::~Scene(){

}

void Scene::setRotateX(float rx){
    this->angle_x = rx;
}

float Scene::getRotateX(){
    return this->angle_x;
}

void Scene::setRotateY(float ry){
    this->angle_y = ry;
}

float Scene::getRotateY(){
    return this->angle_y;
}

void Scene::setRotateZ(float rz){
    this->angle_z = rz;
}

float Scene::getRotateZ(){
    return this->angle_z;
}     

void Scene::setRotation(float rx, float ry, float rz){
    angle_x = rx;
    angle_y = ry;
    angle_z = rz;
}

Point Scene::getRotation(){
    return Point(angle_x, angle_y, angle_z);
}

void Scene::rotate(){
    glRotatef(angle_x, 1.0, 0.0, 0.0);
    glRotatef(angle_y, 0.0, 1.0, 0.0);
    glRotatef(angle_z, 0.0, 0.0, 1.0);
}

void Scene::setTranslateX(float tx){
    this->translate_x = tx;
}

float Scene::getTranslateX(){
    return this->translate_x;
}

void Scene::setTranslateY(float ty){
    this->translate_y = ty;
}

float Scene::getTranslateY(){
    return this->translate_y;
}

void Scene::setTranslateZ(float tz){
    this->translate_z = tz;
}

float Scene::getTranslateZ(){
    return this->translate_z;
}  


void Scene::setTranslation(float tx, float ty, float tz){
    translate_x = tx;
    translate_y = ty;
    translate_z = tz;
}

Point Scene::getTranslation(){
    return Point(translate_x, translate_y, translate_z);
}

void Scene::setScaleX(float sx){
    this->scale_x = sx;
}

void Scene::translate(){
    glTranslatef(translate_x, translate_y, translate_z);
}

float Scene::setScaleX(){
    return this->scale_x;
}

void Scene::setScaleY(float sy){
    this->scale_y = sy;
}

float Scene::setScaleY(){
    return this->scale_y;
}

void Scene::setScaleZ(float sz){
    this->scale_z = sz;
}

float Scene::setScaleZ(){
    return this->scale_z;
}                    


Point Scene::getScenePoint(Point cursor){
    GLdouble x, y, z;
    int viewportMatrix[4];
    double modelview[16];
    double projection[16];
    float p[1];    
    
    
    glGetIntegerv(GL_VIEWPORT, viewportMatrix);
    glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
    glGetDoublev(GL_PROJECTION_MATRIX, projection);
    
    glReadPixels(cursor.getX(), viewportMatrix[3] - cursor.getY(), 
                 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &p);
    
    gluUnProject(cursor.getX(), viewportMatrix[3] - cursor.getY(), p[0], 
                 modelview, projection, viewportMatrix, &x, &y, &z);
                
    return Point(x,y,z);
}        

vector<Selected> Scene::parseSelectedBuffer(GLuint selectBuff[SELECT_BUFFER_LENGTH]){
    vector<Selected> selected;
    for (int i = 0; i < SELECT_BUFFER_LENGTH; i+=4){
        if (selectBuff[i] == 1){
            Selected s;
            s.count = selectBuff[i];
            s.zmin = selectBuff[i+1];
            s.zmax = selectBuff[i+2];
            s.id = selectBuff[i+3];
            selected.push_back(s);
        }
        else{
            break;
        }
    }            
    return selected;
}

vector<Selected> Scene::selectElements(int xPos, int yPos){
    printf("Aqui\n");
    GLfloat fAspect;
    GLuint selectBuff[SELECT_BUFFER_LENGTH];
    GLint hits, viewportMatrix[4];

    glSelectBuffer(SELECT_BUFFER_LENGTH, selectBuff);
    glGetIntegerv(GL_VIEWPORT, viewportMatrix);

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();

    glRenderMode(GL_SELECT);
    glLoadIdentity();
    gluPickMatrix(xPos, viewportMatrix[3] - yPos + viewportMatrix[1], 2,2, viewportMatrix);

    fAspect = (float)viewportMatrix[2] / (float)viewportMatrix[3];
    
    Camera camera = canvas->getCamera();
    Window w = camera.getWindow();
    
    glOrtho(w.left, w.right, w.bottom, w.top, w.near, w.far);	
    gluPerspective( camera.fov , camera.aspect, camera.window.near, camera.window.far);
    gluLookAt( camera.position.getX(), camera.position.getY(), camera.position.getZ(),
               camera.lookat.getX(),   camera.lookat.getY(),   camera.lookat.getZ(),
               camera.up.getX(),       camera.up.getY(),       camera.up.getZ() );

    draw();
    
    hits = glRenderMode(GL_RENDER);
    printf("hits: %i\n", hits);
    
    vector<Selected> selected = parseSelectedBuffer(selectBuff);

    glMatrixMode(GL_PROJECTION);
    glPopMatrix();

    glMatrixMode(GL_MODELVIEW);
    
    return selected;
}        


int Scene::getVision(){
    return vision;
}
