#ifndef SCENE_H
#define SCENE_H

#include <vector>
#include <map>

#include <GL/gl.h>

#include "primitive.h"
#include "graphicobject.h"

using namespace std;

#define SELECT_BUFFER_LENGTH 64

class Canvas;

class Selected{
    public:
        int count;
        int zmin;
        int zmax;
        int id;

        Selected();
        ~Selected();

        void show();
};

class Scene{
    protected:
        Canvas* canvas;
        int vision;
        
        float angle_x;
        float angle_y;
        float angle_z;
        
        float translate_x;
        float translate_y;
        float translate_z;

        float scale_x;
        float scale_y;
        float scale_z;
        
    public:
        Scene(Canvas* canvas);
        ~Scene();
    
        void setRotateX(float rx);
        float getRotateX();
        void setRotateY(float ry);
        float getRotateY();
        void setRotateZ(float rz);
        float getRotateZ();
        void setRotation(float rx, float ry, float rz);
        Point getRotation();
        
        void rotate();
        
        void setTranslateX(float tx);
        float getTranslateX();
        void setTranslateY(float ty);
        float getTranslateY();
        void setTranslateZ(float tz);
        float getTranslateZ();
        void setTranslation(float tx, float ty, float tz);
        Point getTranslation();
        
        void translate();
        
        void setScaleX(float sx);
        float setScaleX();
        void setScaleY(float sy);
        float setScaleY();
        void setScaleZ(float sz);
        float setScaleZ();     
        
        Point getScenePoint(Point cursor);
        vector<Selected> parseSelectedBuffer(GLuint selectBuff[SELECT_BUFFER_LENGTH]);
        vector<Selected> selectElements(int xPos, int yPos);
        virtual int ProcessPick(GLuint selectBuff[SELECT_BUFFER_LENGTH]){}
        
        virtual void addObject(GraphicObject* object){}

        virtual void setVision(int _v){}
        int getVision();
        
        virtual void drawEnviroment(){}
        virtual void drawObject(int id){}
        virtual void drawObjects(){}
        virtual void draw(){}
};

#endif /* SCENE_H */
