#include "trace.h"

Trace::Trace(){
    this->id = 0; 
}

Trace::Trace(int id){
    this->id = id;
}

Trace::~Trace(){
    points.clear();
}

bool Trace::ccw(Point PA, Point PB, Point PC){
    double result = ( ( PB.getX() - PA.getX() ) * ( PC.getZ() - PA.getZ() ) ) 
                  - ( ( PC.getX() - PA.getX() ) * ( PB.getZ() - PA.getZ() ) );
    if (result > 0){
        return true;
    }
    else{
        return false;
    }
}


void Trace::setId(int id){
    this->id = id;
}

int Trace::getId(){
    return this->id;        
}

bool Trace::isEmpty(){
    return this->points.empty();
}

unsigned int Trace::size(){
    return this->points.size();
}

void Trace::erase(int i){
    this->points.erase(this->points.begin()+i);
}     

void Trace::erase(int b, int e){
    if ( b >= 0 and b < e and e < this->points.size())
    for (int i = b; i < e; i++){
        this->points.erase(this->points.begin()+b);
    }
}              

void Trace::clear(){
    this->points.clear();
}

Point& Trace::operator[](int i){
    if (i >= 0){
        return this->points[i];
    }
    else{
        return this->points[this->size() + i];
    }
}

void Trace::addPoint(Point c){
    this->points.push_back(c);
}

void Trace::setPoints(vector<Point> _points){
    this->points = _points;
}

vector<Point>& Trace::getPoints(){
    return this->points;
}

void Trace::filter(float threshold){
    if (points.size() >= 2){
        vector<Point> pts;
        pts.push_back( points.back() );
        for (int i=0; i<points.size(); i++){
            if (pts.back().distance(points[i]) > threshold){
                pts.push_back(points[i]);
            }
        }
        points.clear();
        points = pts;
    }
}

void Trace::upsample(int density){
    vector<Point> pts;
    for (int i=0; i<points.size()-1  &&  points.size() > 2; i++){
        pts.push_back(points[i]);
        for (int j = 1; j <= density; j++){
            float t = float(j) / float (density);
            pts.push_back(points[i]*(1-t) + points[i+1]*t);
        }
    }
    points.clear();
    points = pts;
}

void Trace::smooth(int level){
    double kernel[] = { 0.0625, 0.25, 0.375, 0.25, 0.0625 };

    for ( int l = 0; l < level; l++ ) {
        vector<Point> new_points( points.size() );
        for ( size_t j=0; j<points.size() && points.size() > 4; j++){
	        Point a(0,0);
	        for ( int k=-2; k<3; k++) {
		        int c = ( points.size() + j + k)%points.size();
		        a+= points[c] * kernel[k + 2];
	        }
	        new_points[j] = a;
        }
        points.clear();
        points = new_points;
    }
}

Point Trace::getCentroide(){
    Point center;
    for (int i = 0; i < points.size(); i++){
        center += points[i];
    }
    return center / float(points.size());
}

void Trace::translate(Point p){
    for (int i = 0; i < points.size(); i++){
        points[i] += p;
    }
}

void Trace::adjust(bool modify){
    Point pmin;
    int j,k;

    for (int i = 1; i < points.size(); i++){
        j = 0; k = 1;
        if (points[i].getX() <= points[i-1].getX()){
            points[i-1].setAtributtes(0.0, 1.0, 0.0, 9);
        
            pmin = points[i-1];
            while( points[i+j].getX()<=points[i-1].getX() and i+j < points.size()){
                points[i+j].setAtributtes(1.0, 1.0, 0.0, 5);
                j++;
                if (pmin.getX() > points[i+j].getX()) pmin = points[i+j];
            }
            
            while( i-k > 0 and points[i-k].getX() >= pmin.getX() ){
                points[i-k].setAtributtes(0.0, 0.0, 1.0, 5);
                k++;
            }
            
            if (modify){
                monotonize(i-k,i+j);
            }
        }
        i += j;
    }
    points.front().setY(0.0);
    points.back().setY(0.0);
}

void Trace::monotonize(int init, int end){
    bool monotonous = false;
    
    float xmin = points[init].getX();
    if (init < 3){
        for (int i = init; i < end; i++){
            if (points[i].getX() < xmin) xmin = points[i].getX();
        }
    }

    float xmax = points[end-1].getX();
    if (end > points.size() - 3){
        for (int i = init; i < end; i++){
            if (points[i].getX() > xmax) xmax = points[i].getX();
        }
    }
    
    float lenght = xmax - xmin;
    float inc = lenght / (end - init);

    while (!monotonous){
        for (int i = init; i < end; i++){
            points[i].setX( xmin + ((i - (init+1)) * inc) );
        }
    
        monotonous = true;
        for (int i = init+1; i < end; i++){
            if (points[i].getX() < points[i-1].getX() ){
                monotonous = false;
                break;
            }
        }
    }
}

// metodo nao testado
void Trace::rotateXY(){
    printf("nao testado\n\n");
    
    Point shadow = points.back() - (points.front());
    shadow = shadow.normalize();
    float angle = atan2( shadow.getY(), shadow.getX() );
    
    for (int i = 1; i < points.size(); i++){
        points[i] -= points[0];
        
        Point pos;
        pos.setX( points[i].getX() * cos(-angle) - points[i].getZ()*sin(-angle) );
        pos.setY( points[i].getY() * sin(-angle) + points[i].getZ()*cos(-angle) );
        pos.setZ( points[i].getZ() );
        points[i] = pos;

        points[i] += Point(points[0].getX(), points[0].getY(), 0.0);
    }
}

void Trace::rotateYZ(){
    float angle = 90.0;
    Point centroide = getCentroide();
    
    for (int i = 0; i < points.size(); i++){
        points[i] -= centroide;
        
        Point pos;
        pos.setX( points[i].getX() );
        pos.setY( points[i].getY() * cos(-angle) - points[i].getZ()*sin(-angle) );
        pos.setZ( points[i].getY() * sin(-angle) + points[i].getZ()*cos(-angle) );
        points[i] = pos;

        points[i] += Point(0.0, centroide.getY(), centroide.getZ());
    }
}

void Trace::rotateXZ(){
    Point shadow = points.back() - (points.front());
    Point centroide = shadow / 2.0;
    centroide.setY(0.0);
    
    shadow = shadow.normalize();
    float angle = atan2( shadow.getZ(), shadow.getX() );
    
    for (int i = 0; i < points.size(); i++){
        points[i] -= centroide;
        
        Point pos;
        pos.setX( points[i].getX() * cos(-angle) - points[i].getZ()*sin(-angle) );
        pos.setY( points[i].getY() );
        pos.setZ( points[i].getX() * sin(-angle) + points[i].getZ()*cos(-angle) );
        points[i] = pos;

        points[i] += Point(centroide.getX(), 0.0, centroide.getZ());
    }
}
