#include "skeleton.h"

///////////////////////
//  GeometricVertex  //
///////////////////////

GeometricVertex::GeometricVertex(){
    position = Point(1000.0, 1000.0, 1000.0);
    vertex1 = NULL;
    vertex2 = NULL;
    height = 2.1;
}

GeometricVertex::GeometricVertex(Point pos){
    position = pos;
    vertex1 = NULL;
    vertex2 = NULL;
    height = 2.1;
}

GeometricVertex::~GeometricVertex(){

}

void GeometricVertex::setTopologicalVertices(TopologicalVertex* top1, TopologicalVertex* top2){
    vertex1 = top1;
    vertex2 = top2;
}

pair<TopologicalVertex*,TopologicalVertex*> GeometricVertex::getTopologicalVertex(){
    pair<TopologicalVertex*,TopologicalVertex*> tops;
    tops.first = vertex1;
    tops.second = vertex2;
    return tops;
}

void GeometricVertex::setPosition(Point pos){
    position = pos;
}

Point GeometricVertex::getPosition(){
    return position;
}

void GeometricVertex::setHeight(float h){
    height = h;
}

float GeometricVertex::getHeight(){
    return height;
}

void GeometricVertex::show(){
    printf("%f  ",height);position.show();
}

bool GeometricVertex::operator==(Point vertex){
    return position == vertex;
}

bool GeometricVertex::operator==(GeometricVertex* vertex){
    return position == vertex->getPosition();
}

/////////////////////////
//  TopologicalVertex  //
/////////////////////////

TopologicalVertex::TopologicalVertex(){
    vertex = NULL;
    height = 2.1;
}

TopologicalVertex::TopologicalVertex(GeometricVertex* v){
    vertex = v;
    height = 2.1;
}

TopologicalVertex::~TopologicalVertex(){}

GeometricVertex* TopologicalVertex::getVertex(){
    return vertex;
}

vector< vector<GeometricVertex*> > TopologicalVertex::getInternalVertices(){
    return internalVertices;
}

int TopologicalVertex::size(){
    int count = 0;
    for (int i = 0; i < internalVertices.size(); i++){
        count += internalVertices[i].size();
    }

    return count;
}

void TopologicalVertex::setHeight(float h){
    height = h;
    vertex->setHeight(h);
}

float TopologicalVertex::getHeight(){
    return height;
}

void TopologicalVertex::setPosition(Point pos){
    vertex->setPosition(pos);
}

Point TopologicalVertex::getPosition(){
    return vertex->getPosition();
}

vector<Point> TopologicalVertex::getAdjacentVertices(){
    vector<Point> adjacents;
    for (int i = 0; i < internalVertices.size(); i++){
        adjacents.push_back(internalVertices[i].back()->getPosition());
    }
    return adjacents;
}

void TopologicalVertex::removeBranch(int i){
    internalVertices.erase(internalVertices.begin()+i);
}

void TopologicalVertex::addBranch(vector<GeometricVertex*> branch){
    internalVertices.push_back(branch);
}

vector<GeometricVertex*>& TopologicalVertex::findBranch(GeometricVertex* vertex){
    for (int i = 0; i < internalVertices.size(); i++){
        for (int j = 0; j < internalVertices[i].size(); j++){
            if (vertex->getPosition() == internalVertices[i][j]->getPosition()){
                return internalVertices[i];
            }
        }
    }
    printf("Warning!!! Vai dar errado\n\n");
}

int TopologicalVertex::getIndexBranch(GeometricVertex* v){
    for (int i = 0; i < internalVertices.size(); i++){
        for (int j = 0; j < internalVertices[i].size(); j++){
            if (v->getPosition() == internalVertices[i][j]->getPosition()){
                return i;
            }
        }
    }
    return -1;
}


void TopologicalVertex::show(){
    printf("\nTop: ");vertex->getPosition().show();
    for (int i = 0; i < internalVertices.size(); i++){
        printf("Branch: %i\n", internalVertices[i].size());
//        for (int j = 0; j < internalVertices[i].size(); j++){
//            internalVertices[i][j]->getPosition().show();
//        }            
    }
}

bool TopologicalVertex::operator==(GeometricVertex* v){
    return vertex == v;
}

bool TopologicalVertex::operator==(TopologicalVertex* v){
    return vertex == v->getVertex();
}

/////////////////////
//  GeometricEdge  //
/////////////////////

GeometricEdge::GeometricEdge(){
    vertices.first = -1;
    vertices.second = -1;
}

GeometricEdge::GeometricEdge(int v1, int v2){
    vertices.first = v1;
    vertices.second = v2;
}

void GeometricEdge::setVertex1Index(int index){
    vertices.first = index;
}

int GeometricEdge::getVertex1Index(){
    return vertices.first;
}


void GeometricEdge::setVertex2Index(int index){
    vertices.second = index;
}

int GeometricEdge::getVertex2Index(){
    return vertices.second;
}

pair<int, int> GeometricEdge::getVerticesIndex(){
    return vertices;
    
}

bool GeometricEdge::operator==(GeometricEdge edge){
    pair<int, int> index = edge.getVerticesIndex();
    if ( (vertices.first == index.first and vertices.second == index.second)
    or   (vertices.first == index.second and vertices.second == index.first) ){
        return true;
    }
    return false;
}


///////////////////////
//  TopologicalEdge  //
///////////////////////
TopologicalEdge::TopologicalEdge(){
    vertex1 = NULL;
    vertex2 = NULL;
}

TopologicalEdge::TopologicalEdge(TopologicalVertex* v1, TopologicalVertex* v2){
    vertex1 = v1;
    vertex2 = v2;
}

TopologicalEdge::~TopologicalEdge(){

}

void TopologicalEdge::setTopologicalVertex1(TopologicalVertex* v1){
    vertex1 = v1;
}

TopologicalVertex* TopologicalEdge::getTopologicalVertex1(){
    return vertex1;
}

void TopologicalEdge::setTopologicalVertex2(TopologicalVertex* v2){
    vertex2 = v2;
}

TopologicalVertex* TopologicalEdge::getTopologicalVertex2(){
    return vertex2;
}

pair<TopologicalVertex*, TopologicalVertex*> TopologicalEdge::getVerticesIndex(){
    pair<TopologicalVertex*, TopologicalVertex*> vertices;
    vertices.first = vertex1;
    vertices.second = vertex2;
}

void TopologicalEdge::addInternalVertex(GeometricVertex* geo, bool back){
    if (back){
        internalVertices.push_back(geo);
    }
    else{
        internalVertices.insert(internalVertices.begin(), geo);
    }
}

vector<GeometricVertex*> TopologicalEdge::getInternalVertices(){
    return internalVertices;
}

void TopologicalEdge::updateDistances(){
    Point last = vertex1->getVertex()->getPosition();
    distances.push_back( internalVertices[0]->getPosition().distance(last) );
    last = internalVertices[0]->getPosition();
    for (int i = 1; i < internalVertices.size(); i++){
        distances.push_back(internalVertices[i]->getPosition().distance(last) + distances.back());
        last = internalVertices[i]->getPosition();
    }
}

// verificar se esta ok
void TopologicalEdge::updateGeometricVertexHeight(){
    updateDistances();
    float lenght = distances.back();
    float h;
    float t;
    for (int i = 0; i < distances.size(); i++){
        t = float(distances[i])/float(lenght);
        h = float(1.0f - t) * float(vertex1->getHeight()) + float(t) * float(vertex2->getHeight());
        internalVertices[i]->setHeight(h);
    }
    
}

bool TopologicalEdge::operator==(TopologicalEdge edge){
    if ( ( edge.vertex1->getVertex()->getPosition() 
           == vertex1->getVertex()->getPosition()
        and edge.vertex2->getVertex()->getPosition() 
          == vertex2->getVertex()->getPosition())
      or ( edge.vertex1->getVertex()->getPosition() 
          == vertex2->getVertex()->getPosition()
        and edge.vertex2->getVertex()->getPosition() 
            == vertex1->getVertex()->getPosition()))
    {
        return true;
    }
    return false;
}

void TopologicalEdge::show(){
    printf("Edge: %i\n",internalVertices.size());
    vertex1->getVertex()->show();
    vertex2->getVertex()->show();
    printf("\n");
//    for (int i = 0; i < internalVertices.size(); i++){
//        internalVertices[i]->show();
//    }
}

/////////////////////////
//  GeometricSkeleton  //
/////////////////////////

GeometricSkeleton::GeometricSkeleton(){

}

GeometricSkeleton::~GeometricSkeleton(){

}

GeometricVertex* GeometricSkeleton::getVertex(int i){
    return vertices[i];
}

map<int, GeometricVertex*> GeometricSkeleton::getVertices(){
    return vertices;
}

int GeometricSkeleton::verticesSize(){
    return vertices.size();
}

void GeometricSkeleton::deleteVertex(int index){
    delete(vertices[index]);
    printf("Size: antes: %i",vertices.size());
    vertices.erase(index);
    printf("  depois: %i\n",vertices.size());
}

void GeometricSkeleton::removeUnusedVertices(){
    vector<int> unusedMark;;
    unusedMark.resize(vertices.size());
    for (int i = 0; i < edges.size(); i++){
        unusedMark[edges[i].getVertex1Index()]++;
        unusedMark[edges[i].getVertex2Index()]++;
    }
    
    bool change = true;
    while(change){
        change = false;
        map<int, GeometricVertex*>::iterator it;
        for(it = vertices.begin(); it != vertices.end(); it++){
            if (unusedMark[it->first] == 0){
                vertices.erase(it->first);
                change = true;
                break;
            }
        }
    }
}

map<int, int> GeometricSkeleton::markVertices(){
    markup.clear();
    map<int, GeometricVertex*>::iterator mark_it;
    for(mark_it = vertices.begin(); mark_it != vertices.end(); mark_it++){
        markup[mark_it->first] = 0;
    }
    for (int i = 0; i < edges.size(); i++){
        markup[edges[i].getVertex1Index()]++;
        markup[edges[i].getVertex2Index()]++;
    }
    return markup;
}

int GeometricSkeleton::edgesSize(){
    return edges.size();
}

GeometricEdge GeometricSkeleton::getEdge(int index){
    return edges[index];
}


void GeometricSkeleton::addEdge(Point v1, Point v2){
    GeometricEdge edge;
    edge.setVertex1Index(-1);
    edge.setVertex2Index(-1);
    
    map<int, GeometricVertex*>::iterator it;
    for(it = vertices.begin(); it != vertices.end(); it++){
        if (*(it->second) == v1){
            edge.setVertex1Index(it->first);
            markup[it->first]++;
        }
        if (*(it->second) == v2){
            edge.setVertex2Index(it->first);
            markup[it->first]++;
        }
    }
    
    bool equal = false;
    if (edges.size() > 0){
        for (int i = 0; i < edges.size(); i++){
            if ((edge.getVertex1Index() != -1 or edge.getVertex2Index() != -1)
            and edge == edges[i] ){
                equal = true;
                break;
            }
        }
    }
    if (!equal){
        int pos = vertices.size();
        if (edge.getVertex1Index() == -1){
            vertices[pos] = new GeometricVertex(v1);
            markup[pos] = 1;
            edge.setVertex1Index(pos);
            pos++;
        }
        if (edge.getVertex2Index() == -1){
            vertices[pos] = new GeometricVertex(v2);
            markup[pos] = 1;
            edge.setVertex2Index(pos);
        }
        edges.push_back(edge);
    }
}

void GeometricSkeleton::deleteEdge(int index){
    edges.erase(edges.begin()+index);
}

void GeometricSkeleton::removeSmallEdges(){
    Point v1, v2;
    Point v3, v4;
    
    bool change = true;
    while (change){
        change = false;
        for (int i = 0; i < edges.size(); i++){
            v1 = vertices[edges[i].getVertex1Index()]->getPosition();
            v2 = vertices[edges[i].getVertex2Index()]->getPosition();
            if (v1.distance(v2) < 0.001 and edges.size() > 1){
                for (int j = 0; j < edges.size(); j++){
                    v3 = vertices[edges[j].getVertex1Index()]->getPosition();
                    v4 = vertices[edges[j].getVertex2Index()]->getPosition();
                    if (v3 == v1 or v3 == v2){
                        edges[j].setVertex1Index(edges[i].getVertex1Index());
                        change = true;
                    }
                    else if (v4 == v1 or v4 == v2){
                        edges[j].setVertex2Index(edges[i].getVertex1Index());
                        change = true;
                    }
                }
                if(change){
                    edges.erase(edges.begin()+i);
                    break;
                }
            }
        }
    }
    removeUnusedVertices();
}

int GeometricSkeleton::findVertexIndex(GeometricVertex* v){
    map<int, GeometricVertex*>::iterator it;
    for(it = vertices.begin(); it != vertices.end(); it++){
        if (it->second == v) return it->first;
    }
    printf("\n\nWarning!!!\nVertice Geometrico nao encontrado\n\n");
    return -1;
}

vector<int> GeometricSkeleton::findEdge(int index){
    vector<int> branches;

    for (int i = 0; i < edges.size(); i++){
        if ( edges[i].getVertex1Index() == index 
        or   edges[i].getVertex2Index() == index ){
            branches.push_back(i);
        }
    }
    return branches;
}

int GeometricSkeleton::findEdge(GeometricVertex* v1, GeometricVertex* v2, bool show){
    GeometricEdge e(findVertexIndex(v1), findVertexIndex(v2));
    for (int i = 0; i < edges.size(); i++){
        if (edges[i] == e) return i;
    }
    if (show){
        printf("\n\nWarning!!!\nAresta nao encontrada\n");
        v1->show();
        vertices[findVertexIndex(v1)]->show();
        v2->show();
        vertices[findVertexIndex(v2)]->show();
        printf("v1: %i   v2: %i\n\n", findVertexIndex(v1),findVertexIndex(v2));
    }
    return -1;
}

void GeometricSkeleton::updateGeometricPoints(){
    points.clear();
    map<int, GeometricVertex*>::iterator it;
    for(it = vertices.begin(); it != vertices.end(); it++){
        if(it->second == NULL){
            printf("Xiiiii\n");
        }
        else{
            points.push_back(it->second->getPosition());
        }
    }
}

Point GeometricSkeleton::getPoint(int i){
    if (points.size() == 0){
        updateGeometricPoints();
    }
    return points[i];
}

vector<Point> GeometricSkeleton::getGeometricPoints(){
    if (points.size() == 0){
        updateGeometricPoints();
    }
    return points;
}

void GeometricSkeleton::showVertices(){
    printf("\n**************************************************************\n");
    printf("Geometric Vertices: %i\n", vertices.size());
    for (int i = 0; i < vertices.size(); i++){
        if (vertices[i]) vertices[i]->show();
        else printf("\n\nNULO: %i\n\n",i);
    }
}

void GeometricSkeleton::showEdges(){
    printf("\n**************************************************************\n");
    printf("Geometric Edges: %i\n",edges.size());
    for (int i = 0; i < edges.size(); i++){
        vertices[edges[i].getVertex1Index()]->show();
        vertices[edges[i].getVertex1Index()]->show();
        printf("\n");
    }
}

///////////////////////////
//  TopologicalSkeleton  //
///////////////////////////

TopologicalSkeleton::TopologicalSkeleton(GeometricSkeleton& geo) : geometric(geo){

}

TopologicalSkeleton::~TopologicalSkeleton(){

}

int TopologicalSkeleton::verticesSize(){
    return vertices.size();
}

TopologicalVertex* TopologicalSkeleton::getVertex(int index){
    return vertices[index];
}

vector<TopologicalVertex*> TopologicalSkeleton::getVertices(){
    return vertices;
}

vector<Point> TopologicalSkeleton::getVerticesPoints(){
    if (verticesPoints.size() == 0){
        for (int i = 0; i < vertices.size(); i++){
            Point p = vertices[i]->getVertex()->getPosition();
            p.setY(vertices[i]->getHeight());
            verticesPoints.push_back(p);
        }
    }
    return verticesPoints;
}

void TopologicalSkeleton::addVertex(TopologicalVertex* vertex){
    vertices.push_back(vertex);
}

void TopologicalSkeleton::deleteVertex(int index){
    delete(vertices[index]);
    vertices.erase(vertices.begin()+index);
}

void TopologicalSkeleton::createVertex(int topIndex){
    TopologicalVertex* top = vertices[topIndex];

    vector<int> branches = geometric.findEdge(top->getGeometricIndex());
    vector<GeometricVertex*> branch;
    int edgeIndex;
    
    int target;
    
    for (int i = 0; i < branches.size(); i++){
        branch.clear();
        edgeIndex = branches[i];
        
        if (geometric.getVertex(geometric.getEdge(edgeIndex).getVertex1Index())->getPosition() == geometric.getVertex(top->getGeometricIndex())->getPosition() ){
            target = geometric.getEdge(edgeIndex).getVertex2Index();
        }
        else{
            target = geometric.getEdge(edgeIndex).getVertex1Index();
        }
        branch.push_back(geometric.getVertex(target));
        
        vector<int> internalEdges = geometric.findEdge(target);
        while (internalEdges.size() == 2){
            if (internalEdges[0] == edgeIndex){
                edgeIndex = internalEdges[1];
            }
            else{
                edgeIndex = internalEdges[0];
            }
            
            // verificar se nao ha uma função que melhore esse acesso
            if ( geometric.getVertex(geometric.getEdge(edgeIndex).getVertex1Index())->getPosition() == geometric.getVertex(target)->getPosition() ){
                target = geometric.getEdge(edgeIndex).getVertex2Index();
            }
            else{
                target = geometric.getEdge(edgeIndex).getVertex1Index();
            }
            branch.push_back(geometric.getVertex(target));
            internalEdges = geometric.findEdge(target);
        }
        vertices[topIndex]->addBranch(branch);
    }
}


void TopologicalSkeleton::selectVertex(){
    vertices.clear();
    
    map<int, int>::iterator it;
    for(it = markup.begin(); it != markup.end(); it++){
        if (it->second == 1 or it->second > 2){
            TopologicalVertex* topVert = new TopologicalVertex(geometric.getVertex(it->first));
            topVert->setGeometricIndex(it->first);
            addVertex(topVert);
        }
    }
}


int TopologicalSkeleton::findVertex(TopologicalVertex* v){
    for (int i = 0; i < vertices.size(); i++){
        if (vertices[i] == v) return i;
    }
    printf("\n\nWarning!!!\nVertice Topologico nao encontrado\n\n");
    return -1;
}

int TopologicalSkeleton::findVertex(GeometricVertex* v){
    for (int i = 0; i < vertices.size(); i++){
        if (vertices[i]->getVertex() == v) return i;
    }
    printf("\n\nWarning!!!\nVertice Topologico nao encontrado\n\n");
    return -1;
}
        


void TopologicalSkeleton::verticesSimplification(){
    bool change = true;
    while(change){
        change = false;
        for (int i = 0; i < vertices.size(); i++){
            if (vertices[i]->getInternalVertices().size() == 1 
            and vertices[i]->size() < 5 ){
                GeometricVertex* neighborGeo = vertices[i]->getInternalVertices()[0].front();
                int e = geometric.findEdge(neighborGeo, vertices[i]->getVertex());
                if (e>=0){
                    geometric.deleteEdge(e);
                    int vInd;
                    vInd = geometric.findVertexIndex(vertices[i]->getVertex());
                    geometric.deleteVertex(vInd);
                    
                    if (vertices[i]->size() > 1){
                        TopologicalVertex* topVert;
                        topVert = new TopologicalVertex(neighborGeo);
                        topVert->setGeometricIndex(vertices[i]->getGeometricIndex());
                        addVertex(topVert);
                        
                        createVertex(vertices.size()-1);
                        vector< vector<GeometricVertex*> > intern;
                        intern = vertices[i]->getInternalVertices();
                        
                        if (neighborGeo == intern[0][0]){
                            intern[0].erase(intern[0].begin());
                        }
                        else{
                            intern[0].erase(intern[0].begin()+(intern[0].size()-1));
                        }
                        vertices[vertices.size()-1]->addBranch(intern[0]);
                    }
                    
                    vInd =findVertex(vertices[i]);
                    delete(vertices[vInd]);
                    vertices.erase(vertices.begin()+vInd);

                    change = true;
                    break;
                }
            }
        }
    }
}

int TopologicalSkeleton::edgesSize(){
    return edges.size();
}

TopologicalEdge& TopologicalSkeleton::getEdge(int index){
    return edges[index];
}

vector<TopologicalEdge> TopologicalSkeleton::getEdges(){
    return edges;
}

vector< pair<Point,Point> > TopologicalSkeleton::getEdgesPoints(){
    if (edgesPoints.size() == 0){
        for (int i = 0; i < edges.size(); i++){
            pair<Point,Point> e;
            e.first = edges[i].getTopologicalVertex1()->getVertex()->getPosition();
            e.second = edges[i].getTopologicalVertex2()->getVertex()->getPosition();
            edgesPoints.push_back(e);
        }
    }
    return edgesPoints;
}

int TopologicalSkeleton::findEdge(GeometricVertex* v1, GeometricVertex* v2){
    TopologicalEdge e1( vertices[findVertex(v1)], vertices[findVertex(v2)]);
    for (int i = 0; i < edges.size(); i++){
        if (e1 == edges[i]) return i;
    }
    return -1;
}

void TopologicalSkeleton::updateEdges(){
    int edgeIndex;
    
    edges.clear();
    for(int i = 0; i < verticesSize(); i++){
        vector< vector<GeometricVertex*> > intern;
        intern = vertices[i]->getInternalVertices();
        for(int j = 0; j < intern.size(); j++){
            edgeIndex = findEdge(vertices[i]->getVertex(), intern[j].back());
            if (edgeIndex == -1){
                edges.push_back( TopologicalEdge( vertices[i],
                                                  vertices[findVertex(intern[j].back())] ) );
                 for (int k = 0; k < intern[j].size(); k++){
                    edges.back().addInternalVertex(intern[j][k]);
                 }
            }
        }
    }
    bool change = true;
    while (change){
        change = false;
        for (int i = 0; i < edgesSize(); i++){
            if (edges[i].getInternalVertices().size() <= 1){
                edges.erase(edges.begin()+i);
                change = true;
                break;
            }
        }
    }
}


void TopologicalSkeleton::markByEdge(){
    map<int, int>::iterator it;
    for(it = markup.begin(); it != markup.end(); it++){
        it->second = 0;
    }
    for(int i = 0; i < geometric.edgesSize(); i++){
        markup[geometric.getEdge(i).getVertex1Index()]++;
        markup[geometric.getEdge(i).getVertex2Index()]++;
    }
}

void TopologicalSkeleton::showVertices(){
    printf("\n**************************************************************\n");
    printf("Topologic Vertices: %i\n", vertices.size());
    for (int i = 0; i < vertices.size(); i++){
        if (vertices[i]) vertices[i]->show();
        else printf("\n\nNULO: %i\n\n",i);
    }
}

void TopologicalSkeleton::showEdges(){
    printf("\n**************************************************************\n");
    printf("Topologic Edges: %i\n",edges.size());
    for (int i = 0; i < edges.size(); i++){
        edges[i].show();
        printf("\n");
    }
}

////////////////
//  Skeleton  //
////////////////

Skeleton::Skeleton() : topologic(geometric){

}

Skeleton::~Skeleton(){

}

void Skeleton::complete(){
//    TopologicalVertex* vertex1;
//    TopologicalVertex* vertex2;
//    for (int i = 0; i < topologicalVertices.size(); i++){
//        printf("%i\t", i);
//        vector< vector<GeometricVertex*> > intern;
//        intern = topologicalVertices[i].first->getInternalVertices();
//        for (int j = 0; j < intern.size(); j++){
//            vertex1 = topologicalVertices[i].first;
//            GeometricVertex* last = intern[j].back();
//            for (int k = 0; k < topologicalVertices.size(); k++){
//                if (topologicalVertices[k].first->getVertex()->getPosition() == last->getPosition()){
//                    vertex2 = topologicalVertices[k].first;
//                    break;
//                }
//            }
//            if (vertex2 == NULL) printf("Warning!!! Vai dar errado\n\n");
//            for (int k = 0; k < intern[j].size(); k++){
//                intern[j][k]->setTopologicalVertices(vertex1, vertex2);
//            }
//        }
//    }
}

GeometricSkeleton& Skeleton::getGeometricSkeleton(){
    return geometric;
}

TopologicalSkeleton& Skeleton::getTopologicalSkeleton(){
    return topologic;
}

int Skeleton::size(){
    return geometric.verticesSize();
}

Point Skeleton::operator[](int i){
    return geometric.getPoint(i);
}

vector<Point> Skeleton::getGeometricPoints(){
    return geometric.getGeometricPoints();
}

GeometricVertex* Skeleton::getGeometricVertex(int index){
    return geometric.getVertex(index);
}

vector< pair<Point,Point> > Skeleton::getTopologicalEdgesPoints(){
    return topologic.getEdgesPoints();
}

vector<Point> Skeleton::getTopologicalPoints(){
    return topologic.getVerticesPoints();
}

void Skeleton::updateGeometricHeights(){
    for (int i = 0; i < topologic.edgesSize(); i++){
        topologic.getEdge(i).updateGeometricVertexHeight();
    }
    geometric.updateGeometricPoints();
}


void Skeleton::addEdge(Point v1, Point v2){
    geometric.addEdge(v1, v2);
}

void Skeleton::finishSkeleton(){
    printf("\n\nINIT\n");
    printf("Geometric Edges: %i\n", geometric.edgesSize());
    printf("Geometric Vertices: %i\n",geometric.verticesSize());
    printf("Topological Edges: %i\n",topologic.edgesSize());
    printf("Topological Vertices: %i\n\n", topologic.verticesSize());  
    
    geometric.removeSmallEdges();
    topologic.selectVertex();

    for (int i = 0; i < topologic.verticesSize(); i++){
        topologic.createVertex(i);
    }

    topologic.verticesSimplification();
    topologic.markByEdge();
    topologic.selectVertex();
    
    
    for (int i = 0; i < topologic.verticesSize(); i++){
        topologic.createVertex(i);
    }
    geometric.updateGeometricPoints();
    topologic.updateEdges();
    
    for (int i = 0; i < topologic.verticesSize(); i++){
        if (topologic.getVertex(i) == NULL) printf("WARNING!!!\n%i TopVert NULL\n",i);
    }
    
    printf("\n\nFINAL\n");
    printf("Geometric Edges: %i\n", geometric.edgesSize());
    printf("Geometric Vertices: %i\n",geometric.verticesSize());
    printf("Topological Edges: %i\n",topologic.edgesSize());
    printf("Topological Vertices: %i\n\n", topologic.verticesSize());  
}

void Skeleton::show(){
//    printf("\n**************************************************************\n");
//    printf("Skeleton: %i\n", topologicalVertices.size());
//    for (int i = 0; i < topologicalVertices.size(); i++){
//        if (topologicalVertices[i].first) topologicalVertices[i].first->show();
//        else printf("\n\nNULO: %i\n\n",i);
//    }
}

void Skeleton::edgeShow(){
//    printf("\n**************************************************************\n");
//    printf("Edges: \n");
//    for (int i = 0; i < topologicalEdges.size(); i++){
////        topologicalEdges[i].getTopologicalVertex1()->getVertex()->show();
////        topologicalEdges[i].getTopologicalVertex2()->getVertex()->show();
//        topologicalEdges[i].show();
//        printf("\n");
//    }
}
