#ifndef TERRAIN_H
#define TERRAIN_H

#include <vector>
#include <cmath>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <cmath>
#include <string>

#include <GL/gl.h>

#include "primitive.h"
#include "landform.h"
#include "trace.h"
#include "terraincontours.h"
#include "graphicobject.h"
#include "sketch.h"
#include "skeleton.h"
#include "texture.h"

#include "a48.h"
#include "surface.h"


using namespace std;

#define BASE        01
#define SILHOUETTE  02


#define POINTS      11
#define LINES       12
#define POLYGONS    13
#define CONTOUR     14
#define BOX         15
#define SKELETON    16
#define CONE        17
#define SHADING     18
#define NORMAL      19


class Terrain;

class TerrainIO{
    public:
        static void saveData(string filename, Terrain* t);
        static void readData(string filename, Terrain* t);
        static void show(Terrain* t);
};

class Terrain : public GraphicObject{
    protected:
        float width;
        float depth;
        float height;


        int rows;
        int cols;

        Sketch contour;
        Point camera;
        
        Dem grid;
        Mesh mesh;
    
        vector< vector<float> > dem;
        vector< vector<Point> > colors;
        
        vector<Landform*> landforms;

        vector<bool> land_mark;
        
        int modeType;
        
        bool showPlane;
        Plane plane;
        
        bool finish_base;
        Base base;
        bool finish_silhouette;
        Trace silhouette;
        
        vector<Point> voronoi;

        bool showMesh;
        bool colorize;
        bool noise;
        bool bumpmapping;

        void init();
        void updateGridLevel(int l);
        
    public:
        int level;

        float landHeight;
        float landSlope;
        
        Terrain();
        Terrain(float w, float d);
        ~Terrain();

        void setShowMesh(bool show);
        bool getShowMesh();

        void setColorize(bool color);
        bool getColorize();

        void setNoise(bool n);
        bool getNoise();

        void setBumpMapping(bool bump);
        bool getBumpMapping();
        
        void clear();
        void plan();

        void changeLevel(int l);
        void changeCamera(Point t, Point r);
        Point getCamera();

        vector< vector<float> >& getDem();
        
        Landform* createLandform(int type);
        
        void startSilhouette();
        void addSilhouettePoint(Point p);
        Trace getSilhouette();
        void finishSilhouette();
        void adjustSilhouette();
        
        void startBase();
        void addBasePoint(Point p);
        Base& getBase();
        void finishBase();
        
        void addLandform(Landform* landform);
        Landform* getLandform(int index = -1);
        Landform* getLandform(Point p);
        void removeLandform(int index = -1);
        
        void setDimensions(float _width, float _depth);
        float getPeak();

        void readData(string filename, bool show = false);
        void writeData(string filename);
        void showData();
        void generatePlane(int rows, int cols);
        void updateTerrain();
        vector< vector<float> > addNoise();
        void updateSurface();
        
        void enablePlane(Point center, Point normal);
        void disablePlane();
        
        void setMode(int m);
        int getMode();
        void switchMode();
        
        void draw(bool showBase, bool showSilhouette, bool showShadow, bool showBox);
        void draw();
        
        void drawMesh();
};

#endif // TERRAIN_H
