#include "terraincontours.h"

#include <QColor>

Base::Base(){
    normals.clear();
    orientation = 0.0;
}

Base::~Base(){

}

// metodo so funciona para o grid da aplicação
// deixar generico
void Base::saveImageBase(){
    DrawCanvas canvas;
    canvas.setGeometry(QRect(0, 0, 100, 100));
    canvas.setPoints(points);
    canvas.setCamera(0.0, 25.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,-1.0,-1.0,-1.0, false);
    canvas.setWindow(0, 25, 0, 25, -10, 10);
    internalGridImage = canvas.saveImage();
}

void Base::calculateSkeleton(Voronoi v){
    if (skeleton.size() == 0){
        vector<Point> vertices = v.getVertex();
        for (int i = 0; i < v.edges.size(); i++){
            if (v.edges[i].v1 >= 0 and v.edges[i].v2 >= 0){
                if (   isInternalPoint(vertices[v.edges[i].v1])
                   and isInternalPoint(vertices[v.edges[i].v2]) )
                {
                    skeleton.addEdge(vertices[v.edges[i].v1], vertices[v.edges[i].v2] );
                }
            }
        }
        skeleton.finishSkeleton();
    }
}

void Base::preprocessing(){
    upsample(20);
    filter(0.05);
    smooth(200);
}

void Base::operator=(Base b){
    this->points = b.getPoints();
}

Skeleton Base::getSkeleton(){
    return skeleton;
}

void Base::finishBase(){
    saveImageBase();
    calculateBaseMaxisDistances();
}

// metodo so funciona para o grid da aplicação
// deixar generico
bool Base::isInternalPoint(Point p){
    if (internalGridImage.width() * internalGridImage.height() == 0 ){
        saveImageBase();
    }
    
    int row = int( 0.5 + ( p.getX() + 12.5 ) * 100.0 / 25.0 );
    int col = int( 0.5 + ( p.getZ() + 12.5 ) * 100.0 / 25.0 );
    
    if (row > 99 or row < 0 or col > 99 or col < 0) return false;
    QColor c = internalGridImage.pixel(row, col);

    if (c.red() < 10){
        return true;
    }
    return false;
}

Point Base::calculateNormal(int index){
    if (normals.size() < points.size() ) calculateNormals();
    
    if ( index >= normals.size() ) return Point(0.0, 0.0, 0.0);
    
    if (normals[index].lenght() < 0.1){
        int s = this->points.size();
        Point normal;
        normal.setX(-(this->points[(index+1)%s] - this->points[(s+index-1)%s]).getZ());
        normal.setY(  this->points[index].getY() );
        normal.setZ( (this->points[(index+1)%s] - this->points[(s+index-1)%s]).getX());
        normal = normal.normalize();
        if (fabs(orientation) < 0.1 ){
            if ( isInternalPoint( this->points[index] + normal*0.1 ) ){
                orientation = 1.0;
            }
            else{
                orientation = -1.0;
            }
        }
        normal *= orientation;
        return normal;
    }
    else{
        return normals[index];
    }
}

void Base::calculateNormals(){
    this->normals.clear();
    for (int i = 0; i < this->points.size(); i++){
        normals.addPoint(Point(0.0, 0.0,0.0));
    }
}


pair<float, int> Base::getBaseMaxisDistance(int index){
    pair<float, int> D;
    D.first = 100.0;
    D.second = -1;
    int cont = 1;
    
    if (baseMaxisDistances.size() < points.size() ) calculateNormals();
    if ( index >= baseMaxisDistances.size() ){
        pair<float, int> ret;
        ret.first = 0.0;
        ret.second = -1;
//        printf("WARNING!!!\nIndex %i - %i out of base\n\n",index, size());
        return ret;
    } 
    if (baseMaxisDistances[index].first < 0.1){
        Point normal = calculateNormal(index);
        normal = normal.normalize();
        
        GeometricSkeleton& geometric_skel = skeleton.getGeometricSkeleton();
        map<int, GeometricVertex*> geometric_vertices;
        geometric_vertices = geometric_skel.getVertices();

        if (geometric_vertices.size() == 0){
            calculateMedialAxis();
        }
        while(D.second == -1){
            map<int, GeometricVertex*>::iterator it;
            for(it = geometric_vertices.begin(); it != geometric_vertices.end(); it++){
                Point m = it->second->getPosition() - this->points[index];
                Point proj = normal * normal.dotProduct(m);
                if( (m-proj).lenght() <  0.02*cont){
                    if (D.second != -1){
                        Point v = m.normalize();
                        if ( m.lenght() < D.first and v.dotProduct(normal) > 0.8 ){
                            D.second = it->first;
                            D.first = this->points[index].distance(it->second->getPosition());
                        }
                    }
                    else{
                        D.second = it->first;
                        D.first = this->points[index].distance(it->second->getPosition());
                    }
                }
            }
            cont++;
            if (cont > 20){
                printf("WARNING!!!\nIndex %i not founded\n\n",index);
                break;
            }        
        }
        baseMaxisDistances[index] = D;
        return D;
    }
    else{
        return baseMaxisDistances[index];
    }
}


void Base::calculateBaseMaxisDistances(){
    this->baseMaxisDistances.clear();
    for (int i = 0; i < this->points.size(); i++){
        pair<float, int> d;
        d.first = 0.0;
        d.second = -1;
        this->baseMaxisDistances.push_back(d);
    }
}

void Base::calculateMedialAxis(){
    if (points.size() > 5){
        Base baseCopy;
        baseCopy.setPoints(this->points);
        baseCopy.upsample(20);
        baseCopy.filter(0.01);
        
        Voronoi v;
        v.setPoints( baseCopy.getPoints() );
        v.createVoronoi();
        calculateSkeleton(v);
    }
}


int Base::projectPointBase(Point p){
    float d = 1000.0;
    int index = -1;
    float dist;
    for (int i = 0; i < this->points.size(); i++){
        dist = this->points[i].distance(p);
        if( dist < d ){
            d = dist;
            index = i;
        }
    }
    return index;
}

void Base::updateBaseMaxisDistance(int index, float D){
    if ( index >= baseMaxisDistances.size() ){
//        printf("WARNING!!!\n(updateBaseMaxisDistance)\nIndex %i - %i out of base\n\n",index, size());
    }
    else{
        if (baseMaxisDistances[index].first < D){
            baseMaxisDistances[index].first = D;
        }
    }
}
