#ifndef TERRAIN_CONTOURS_H
#define TERRAIN_CONTOURS_H

#include <vector>

#include <QString>
#include <QImage>

#include "primitive.h"
#include "trace.h"
#include "maxis.h"

#include "canvas.h"
#include "skeleton.h"

class Base : public Trace{
    private:
        Skeleton skeleton;
        QImage internalGridImage;
        Trace normals;
        float orientation;
        vector< pair<float, int> > baseMaxisDistances;
        

        void calculateSkeleton(Voronoi v);
        void preprocessing();
        
    public:
        Base();
        ~Base();
        
        void operator=(Base b);
        Skeleton getSkeleton();
        
        void finishBase();
                
        void saveImageBase();
        bool isInternalPoint(Point p);
        Point calculateNormal(int index);
        void calculateNormals();
        pair<float, int> getBaseMaxisDistance(int index);
        void calculateBaseMaxisDistances();
        void calculateMedialAxis();
        int projectPointBase(Point p);

        void updateBaseMaxisDistance(int index, float D);
        void updateBaseMaxisDistances(){
            for (int i = 1; i < baseMaxisDistances.size()-1; i++){
                if (fabs(baseMaxisDistances[i].first - baseMaxisDistances[i-1].first) > 0.1
                or  fabs(baseMaxisDistances[i].first - baseMaxisDistances[i+1].first) > 0.1 ){
                    baseMaxisDistances[i].first = (baseMaxisDistances[i-1].first + baseMaxisDistances[i+1].first) / 2.0;
                }
            }
        }

        void showBMD(){
            for (int i = 1; i < baseMaxisDistances.size()-1; i++){
                if (fabs(baseMaxisDistances[i].first - baseMaxisDistances[i-1].first) > 0.1
                or  fabs(baseMaxisDistances[i].first - baseMaxisDistances[i+1].first) > 0.1 ){
                    baseMaxisDistances[i].first = (baseMaxisDistances[i-1].first + baseMaxisDistances[i+1].first) / 2.0;
                }
                printf("%ith  %f\n", i, baseMaxisDistances[i].first);
            }
        }

        int showPointBase(){
            for (int i = 0; i < this->points.size(); i++){
                this->points[i].show();
            }
        }
};

class Silhouette : public Trace{
    public:
        Silhouette(){}
        ~Silhouette(){}

};

#endif // TERRAIN_CONTOURS_H
