#include "texture.h"

////////////////////
//  LandColorize  //
////////////////////

LandColorize::LandColorize(){
    rows = 0;
    cols = 0;

    width = 0.0;
    depth = 0.0;

    widthInc = 0.0;
    depthInc = 0.0;
}

LandColorize::~LandColorize(){

}

////////////////////
//  HillColorize  //
////////////////////

void HillColorize::setupColorMatrix(float w, float d, vector< vector<float> > dem){
    rows = dem.size();
    cols = dem[0].size();

    width = w;
    depth = d;

    widthInc = width/float(rows);
    depthInc = depth/float(cols);

    colorMatrix.resize(rows);
    for (int i = 0; i < rows; i++){
        colorMatrix[i].resize(cols);
        for (int j = 0; j < cols; j++){
            if (dem[i][j] < 0.1)
                colorMatrix[i][j] = Point (0, 0, 150 + (rand() % 10)*10);
            else if (dem[i][j] + sin(float(i))*cos(float(j)) < 2.0)
                 colorMatrix[i][j] = Point(175  + (rand() % 20), 145 + (rand() % 20), 101  + (rand() % 20));
            else if (dem[i][j] - sin(PI/float(i))*cos(float(i-j)*PI/3.0) < 5.0)
                colorMatrix[i][j] = Point(124  + (rand() % 20), 252,  + (rand() % 50));
            else if (dem[i][j] + sin(float(i)*PI/10.0)*cos(float(i+j)*PI/17.0) < 8.0)
                colorMatrix[i][j] = Point(85  + (rand() % 20), 107  + (rand() % 20), 47  + (rand() % 20));
            else
                colorMatrix[i][j] = Point (230  + (rand() % 20), 230  + (rand() % 20), 230  + (rand() % 20));
        }
    }
}

Point HillColorize::getColor(Point p){
//    if (colorMatrix.size() > 0){
//        int r = ( (width / 2.0)  + p.getX() ) / widthInc;
//        int c = ( (depth / 2.0) + p.getZ() ) / depthInc;
//        return colorMatrix[(r<rows-1?r:rows-1)][(c<cols-1?c:cols-1)];
//    }
//    return Point(0.0, 0.0, 0.0);

//    if (p.getY() < 0.1)
//        return Point(0, 0, 150 + (rand() % 10)*10);
//    else
    if (p.getY() + sin(p.getX())*cos(p.getZ()) < 2.0)
         return Point(175  + (rand() % 20), 145 + (rand() % 20), 101  + (rand() % 20));
    else if (p.getY() - sin(PI/p.getX())*cos((p.getX()-p.getZ())*PI/3.0) < 5.0)
        return Point(124  + (rand() % 20), 252,  + (rand() % 50));
    else if (p.getY() + sin(p.getX()*PI/10.0)*cos((p.getX()+p.getZ())*PI/17.0) < 8.0)
        return Point(85  + (rand() % 20), 107  + (rand() % 20), 47  + (rand() % 20));
    else
        return Point(230  + (rand() % 20), 230  + (rand() % 20), 230  + (rand() % 20));
}


//////////////////////
//  PlateuColorize  //
//////////////////////

void PlateuColorize::setupColorMatrix(float w, float d, vector< vector<float> > dem){
    rows = dem.size();
    cols = dem[0].size();

    width = w;
    depth = d;

    widthInc = width/float(rows);
    depthInc = depth/float(cols);

    colorMatrix.resize(rows);
    for (int i = 0; i < rows; i++){
        colorMatrix[i].resize(cols);
        for (int j = 0; j < cols; j++){
            if (dem[i][j] < 0.1)
                colorMatrix[i][j] = Point (0, 0, 150 + (rand() % 10)*10);
            else if (dem[i][j] + sin(float(i))*cos(float(j)) < 2.0)
                 colorMatrix[i][j] = Point(175  + (rand() % 20), 145 + (rand() % 20), 101  + (rand() % 20));
            else if (dem[i][j] - sin(PI/float(i))*cos(float(i-j)*PI/3.0) < 5.0)
                colorMatrix[i][j] = Point(124  + (rand() % 20), 252,  + (rand() % 50));
            else if (dem[i][j] + sin(float(i)*PI/10.0)*cos(float(i+j)*PI/17.0) < 8.0)
                colorMatrix[i][j] = Point(85  + (rand() % 20), 107  + (rand() % 20), 47  + (rand() % 20));
            else
                colorMatrix[i][j] = Point (230  + (rand() % 20), 230  + (rand() % 20), 230  + (rand() % 20));
        }
    }
}

Point PlateuColorize::getColor(Point p){
    if (p.getY() < 0.9 * peak)
         return Point(175  + (rand() % 20), 145 + (rand() % 20), 101  + (rand() % 20));
    else
        return Point(85  + (rand() % 20), 107  + (rand() % 20), 47  + (rand() % 20));

}

///////////////////////
//  VolcanoColorize  //
///////////////////////

void VolcanoColorize::setupColorMatrix(float w, float d, vector< vector<float> > dem){
    rows = dem.size();
    cols = dem[0].size();

    width = w;
    depth = d;

    widthInc = width/float(rows);
    depthInc = depth/float(cols);

    colorMatrix.resize(rows);
    for (int i = 0; i < rows; i++){
        colorMatrix[i].resize(cols);
        for (int j = 0; j < cols; j++){
            if (dem[i][j] < 0.1)
                colorMatrix[i][j] = Point (0, 0, 150 + (rand() % 10)*10);
            else if (dem[i][j] + sin(float(i))*cos(float(j)) < 2.0)
                 colorMatrix[i][j] = Point(175  + (rand() % 20), 145 + (rand() % 20), 101  + (rand() % 20));
            else if (dem[i][j] - sin(PI/float(i))*cos(float(i-j)*PI/3.0) < 5.0)
                colorMatrix[i][j] = Point(124  + (rand() % 20), 252,  + (rand() % 50));
            else if (dem[i][j] + sin(float(i)*PI/10.0)*cos(float(i+j)*PI/17.0) < 8.0)
                colorMatrix[i][j] = Point(85  + (rand() % 20), 107  + (rand() % 20), 47  + (rand() % 20));
            else
                colorMatrix[i][j] = Point (230  + (rand() % 20), 230  + (rand() % 20), 230  + (rand() % 20));
        }
    }
}

Point VolcanoColorize::getColor(Point p){
    if (p.getY() + sin(p.getX()) * cos(p.getZ() + p.getX()) < 1.0)
        return Point(85  + (rand() % 20), 107  + (rand() % 20), 47  + (rand() % 20));
    else if (p.getY()+ cos(p.getZ()) * cos(3 * p.getY() - p.getX()) < 0.7 * peak)
         return Point(146  + (rand() % 20), 90 + (rand() % 20), 30  + (rand() % 20));
    else
        return Point(230  + (rand() % 20), 80  + (rand() % 20), 20  + (rand() % 20));

}

//////////////////////
//  IslandColorize  //
//////////////////////

void IslandColorize::setupColorMatrix(float w, float d, vector< vector<float> > dem){
    rows = dem.size();
    cols = dem[0].size();

    width = w;
    depth = d;

    widthInc = width/float(rows);
    depthInc = depth/float(cols);

    colorMatrix.resize(rows);
    for (int i = 0; i < rows; i++){
        colorMatrix[i].resize(cols);
        for (int j = 0; j < cols; j++){
            if (dem[i][j] < 0.1)
                colorMatrix[i][j] = Point (0, 0, 150 + (rand() % 10)*10);
            else if (dem[i][j] + sin(float(i))*cos(float(j)) < 2.0)
                 colorMatrix[i][j] = Point(175  + (rand() % 20), 145 + (rand() % 20), 101  + (rand() % 20));
            else if (dem[i][j] - sin(PI/float(i))*cos(float(i-j)*PI/3.0) < 5.0)
                colorMatrix[i][j] = Point(124  + (rand() % 20), 252,  + (rand() % 50));
            else if (dem[i][j] + sin(float(i)*PI/10.0)*cos(float(i+j)*PI/17.0) < 8.0)
                colorMatrix[i][j] = Point(85  + (rand() % 20), 107  + (rand() % 20), 47  + (rand() % 20));
            else
                colorMatrix[i][j] = Point (230  + (rand() % 20), 230  + (rand() % 20), 230  + (rand() % 20));
        }
    }
}

Point IslandColorize::getColor(Point p){
    if (p.getY() < 0.01)
        return Point(0, 0, 150 + (rand() % 10)*10);
    else if (p.getY() < 0.5)
        return Point(230  + (rand() % 20), 230  + (rand() % 20), 190  + (rand() % 20));
    else
        return Point(81  + (rand() % 20), 81  + (rand() % 20), (rand() % 20));

}
