#ifndef CANVAS_H
#define CANVAS_H

#include <vector>
#include <cstdio>

#include <QGLWidget>
#include <QKeyEvent>
#include <QPainter>
#include <GL/gl.h>

#include "camera.h"
#include "primitive.h"
#include "trace.h"
#include "scene.h"

using namespace std;

#define UP    10
#define DOWN  11
#define FRONT 12
#define BACK  13
#define LEFT  14
#define RIGHT 15

class Canvas : public QGLWidget{
    protected:
        Camera camera;
        Scene* scene;
        
        string title;
    
        int viewWidth;
        int viewHeight;
        
        float windLeft;
        float windRight;
        float windBottom;
        float windTop;
        
        float angle_x;
        float angle_y;
        float angle_z;
        
        Point translation;
        
        int idTimer;
        int interval;
        
        void timerEvent(QTimerEvent *event);

    public:
        Canvas(QWidget* parent);
        ~Canvas();
        
        void setRotation(float x, float y, float z);
        void setTranslation(float x, float y, float z);

        void setCamera( float posX, float posY, float posZ, 
                        float latX, float latY, float latZ, 
                        float upX,  float upY,  float upZ,
                        float fov=-1.0, float aspect=-1.0, bool perspective=false);
        Camera getCamera();
                        
        void setWindow(float l, float r, float b, float t, float n, float f);
        void setViewport(int _left, int _top, int _width, int _height);

        void enableFocus();
        
        void setTitle(string _title);
        
        void setScene(Scene* s);
        Scene* getScene();
        
        virtual void initializeGL(){}
        virtual void resizeGL(int _width, int _height);
};

class SceneCanvas : public Canvas{
    protected:
        int vision;

    public:
        SceneCanvas(QWidget* parent);
        ~SceneCanvas();
        
//        void setVision(int _v);
//        void changeVision(int _v);
        
        void initializeGL();
};

class ImageCanvas : public Canvas{
    protected:
        QImage image;

    public:
        ImageCanvas(QWidget* _par = NULL);
        ~ImageCanvas();
        
        virtual void updateImage(){}

        void initializeGL();
        void paintGL();
};

class SketchCanvas : public Canvas{
    protected:
        Trace trace;

    public:
        SketchCanvas(QWidget* _par);
        ~SketchCanvas();
        
        virtual void updateSketch(){}

        void initializeGL();
        void paintGL();
};

class DrawCanvas : public ImageCanvas{
    protected:
        vector<Point> points;
        int mode;
        
    public:
        DrawCanvas(QWidget* _par = NULL);
        ~DrawCanvas();
        
        QImage saveImage();

        void setPoints(vector<Point> pts){
            points = pts;
        }
        
        void setMode(int m){
            mode = m;
        }

        void initializeGL();
        void paintGL();
};


#endif /* CANVAS_H */
