#ifndef INTERPRETER_H
#define INTERPRETER_H

#include <vector>

#include <GL/gl.h>

#include "primitive.h"
#include "terrain.h"
#include "mycanvas.h"

#include "suggestionaction.h"

using namespace std;

class MySuggestion;
class MyAnnotation;

class Interpreter{
    protected:
        int viewWidth;
        int viewHeight;
        
        float windLeft;
        float windRight;
        float windBottom;
        float windTop;
    
        vector<Point> cursors;
        Terrain& terrain;
        Landform* land;
        
        MySuggestion* suggestion;
        ModelingCanvas* canvas;
        
        map<int, Action*> actions;
        
        void executeAction(int i);
        
        void drawInvisiblePlane(Point first, Point last);
        Point projectionPoint(Point point);
        
    public:
        Interpreter(MySuggestion* sug, Terrain& _terrain);
        ~Interpreter();
        
        void addAction(int key, Action* action);
        
        void setViewport(int _width, int _height);
        void setWindow();
        void setCanvas(ModelingCanvas* c){
            canvas = c;
        };
        
        virtual void startTrace(){};
        virtual void addCursor(Point c){};
        virtual void finishTrace(){};
};

class UpInterpreter : public Interpreter{
    private:
        MyAnnotation* annotation;

    public:

        bool showHill;
        bool showPlateu;
        bool showVolcano;
        bool showIsland;

        UpInterpreter(MySuggestion* sug, Terrain& _terrain);
        ~UpInterpreter();
        
        void setAnnotation(MyAnnotation* note);
        
        void startTrace();
        void addCursor(Point c);
        void finishTrace();
};

class FrontInterpreter : public Interpreter{
    private:
        Selected selected;
        bool moving;
        Point oldVertex;
//        Point newVertex;
        TopologicalVertex* top;
        
        bool processPick(Point cursor);
        void moveTopologicalPoint();
        
        
    public:
        FrontInterpreter(MySuggestion* sug, Terrain& _terrain);
        ~FrontInterpreter();
        
        void startTrace();
        void addCursor(Point c);
        void finishTrace();
};

#endif /* INTERPRETER_H */
