#include "mainwindow.h"
#include "interpreter.h"

MainWindow::MainWindow(QMainWindow *parent) : QMainWindow(parent){
    this->resize(1550, 1130);
    
    QMenu* fileMenu = menuBar()->addMenu(tr("&File"));

    QAction* newAct = new QAction(tr("&New"), this);
    newAct->setShortcut(tr("Ctrl+N"));
    connect(newAct, SIGNAL(triggered()), this, SLOT(newFile()));
    fileMenu->addAction(newAct);

    QAction* openFileAct = new QAction(tr("&Open"), this);
    openFileAct->setShortcut(tr("Ctrl+O"));
    connect(openFileAct, SIGNAL(triggered()), this, SLOT(openFile()));
    fileMenu->addAction(openFileAct);

    QAction *saveFileAct = new QAction(tr("&Save File"), this);
    saveFileAct->setShortcut(tr("Ctrl+S"));
    connect(saveFileAct, SIGNAL(triggered()), this, SLOT(saveFile()));
    fileMenu->addAction(saveFileAct);

    QAction* saveImageAct = new QAction(tr("&Save Image"), this);
    saveImageAct->setShortcut(tr("Ctrl+S"));
    connect(saveImageAct, SIGNAL(triggered()), this, SLOT(saveImage()));
    fileMenu->addAction(saveImageAct);

    QAction* saveDemImageAct = new QAction(tr("&Save DEM Image"), this);
    saveDemImageAct->setShortcut(tr("Ctrl+S"));
    connect(saveDemImageAct, SIGNAL(triggered()), this, SLOT(saveDemImage()));
    fileMenu->addAction(saveDemImageAct);

    QAction* saveConeAct = new QAction(tr("&Save Cone Image"), this);
    saveConeAct->setShortcut(tr("Ctrl+S"));
    connect(saveConeAct, SIGNAL(triggered()), this, SLOT(saveCone()));
    fileMenu->addAction(saveConeAct);

    QAction* quitAct = new QAction(tr("&Quit"), this);
    quitAct->setShortcut(tr("Ctrl+Q"));
    connect(quitAct, SIGNAL(triggered()), qApp, SLOT(quit()));
    fileMenu->addAction(quitAct);

    QWidget* centralWidget = new QWidget(this);
    setCentralWidget(centralWidget);
    
    QPushButton* wireButtom = new QPushButton("W",centralWidget);
    wireButtom->setGeometry(QRect(330, 815, 40, 40));
    connect(wireButtom, SIGNAL(pressed()), this, SLOT(setWireMode()));
    
    QPushButton* prButtom = new QPushButton("PR",centralWidget);
    prButtom->setGeometry(QRect(380, 815, 40, 40));
    connect(prButtom, SIGNAL(pressed()), this, SLOT(setPrMode()));
    
    QPushButton* nprButtom = new QPushButton("NPR",centralWidget);
    nprButtom->setGeometry(QRect(430, 815, 40, 40));
    connect(nprButtom, SIGNAL(pressed()), this, SLOT(setNprMode()));
    
    QPushButton* contourButtom = new QPushButton("Cont",centralWidget);
    contourButtom->setGeometry(QRect(480, 815, 40, 40));
    connect(contourButtom, SIGNAL(pressed()), this, SLOT(setContourMode()));
    
    QPushButton* boxButtom = new QPushButton("Box",centralWidget);
    boxButtom->setGeometry(QRect(530, 815, 40, 40));
    connect(boxButtom, SIGNAL(pressed()), this, SLOT(setBoxMode()));
    
    QPushButton* skeletonButtom = new QPushButton("Skel",centralWidget);
    skeletonButtom->setGeometry(QRect(580, 815, 40, 40));
    connect(skeletonButtom, SIGNAL(pressed()), this, SLOT(setSkeletonMode()));

    QPushButton* conesButtom = new QPushButton("Cone",centralWidget);
    conesButtom->setGeometry(QRect(630, 815, 40, 40));
    connect(conesButtom, SIGNAL(pressed()), this, SLOT(setConeMode()));

    QPushButton* shadignButtom = new QPushButton("Shade",centralWidget);
    shadignButtom->setGeometry(QRect(680, 815, 40, 40));
    connect(shadignButtom, SIGNAL(pressed()), this, SLOT(setShadingMode()));




    hill = new QCheckBox("Generate Hill",centralWidget);
    hill->setGeometry(QRect(750, 815, 130, 25));
    connect(hill, SIGNAL(released()), this, SLOT(setShowHill()));

    plateu = new QCheckBox("Generate Plateu",centralWidget);
    plateu->setGeometry(QRect(750, 840, 130, 25));
    connect(plateu, SIGNAL(released()), this, SLOT(setShowPlateu()));

    volcano = new QCheckBox("Generate Volcano",centralWidget);
    volcano->setGeometry(QRect(890, 815, 140, 25));
    connect(volcano, SIGNAL(released()), this, SLOT(setShowVolcano()));

    island = new QCheckBox("Generate Island",centralWidget);
    island->setGeometry(QRect(890, 840, 130, 25));
    connect(island, SIGNAL(released()), this, SLOT(setShowIsland()));




    QLabel* heightLabel = new QLabel("Height", centralWidget);
    heightLabel->setGeometry(QRect(1050, 820, 50, 15));
    heightLabel->setAlignment(Qt::AlignRight);

    heightBar = new QScrollBar(Qt::Horizontal, centralWidget);
    heightBar->setGeometry(QRect(1100, 820, 200, 15));
    connect(heightBar, SIGNAL(sliderReleased()), this, SLOT(setLandHeight()));

    QLabel* slopeLabel = new QLabel("Slope", centralWidget);
    slopeLabel->setGeometry(QRect(1050, 840, 50, 15));
    slopeLabel->setAlignment(Qt::AlignRight);

    slopeBar = new QScrollBar(Qt::Horizontal, centralWidget);
    slopeBar->setGeometry(QRect(1100, 840, 200, 15));
    connect(slopeBar, SIGNAL(sliderReleased()), this, SLOT(setLandSlope()));




    showMesh = new QCheckBox("Show Mesh",centralWidget);
    showMesh->setGeometry(QRect(1300, 815, 100, 25));
    connect(showMesh, SIGNAL(released()), this, SLOT(checkShowMesh()));

    colorize = new QCheckBox("Colorize",centralWidget);
    colorize->setGeometry(QRect(1300, 840, 100, 25));
    connect(colorize, SIGNAL(released()), this, SLOT(checkColorize()));

    noise = new QCheckBox("Noise",centralWidget);
    noise->setGeometry(QRect(1400, 815, 100, 25));
    connect(noise, SIGNAL(released()), this, SLOT(checkNoise()));

    bumpmapping = new QCheckBox("Bump Mapping",centralWidget);
    bumpmapping->setGeometry(QRect(1400, 840, 130, 25));
    connect(bumpmapping, SIGNAL(released()), this, SLOT(checkBumpMapping()));



    terrain = new Terrain(25, 25);
    terrain->generatePlane(600,600);
    
    suggestion = new MySuggestion(280, 185, centralWidget);
    suggestion->setGeometry(QRect(10, 865, 1520, 230));
    suggestion->setBackgroundRole(QPalette::Dark);
    suggestion->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
    suggestion->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    suggestion->addAction(HAROLD_GEN, new GenerateHaroldAction(suggestion));
    suggestion->addAction(DISTANCE_HILL_GEN, new GenerateDistanceHillAction(suggestion));
    suggestion->addAction(MAXISDISTANCE_HILL_GEN, new GenerateMaxisDistanceHillAction(suggestion));
    suggestion->addAction(NORMALIZED_HILL_GEN, new GenerateNormalizedHillAction(suggestion));
    suggestion->addAction(STEP_PLATEU_GEN, new GenerateStepPlateuAction(suggestion));
    suggestion->addAction(RAMP_PLATEU_GEN, new GenerateRampPlateuAction(suggestion));
    suggestion->addAction(SMOOTH_PLATEU_GEN, new GenerateSmoothPlateuAction(suggestion));
    suggestion->addAction(MAXIS_PLATEU_GEN, new GenerateMaxisPlateuAction(suggestion));
    suggestion->addAction(VOLCANO_GEN, new GenerateVolcanoAction(suggestion));
    suggestion->addAction(ISLAND_GEN, new GenerateIslandAction(suggestion));

    suggestion->addAction(HAROLD_EXE, new ExecuteHaroldAction(suggestion));
    suggestion->addAction(DISTANCE_HILL_EXE, new ExecuteDistanceHillAction(suggestion));
    suggestion->addAction(MAXISDISTANCE_HILL_EXE, new ExecuteMaxisDistanceHillAction(suggestion));
    suggestion->addAction(NORMALIZED_HILL_EXE, new ExecuteNormalizedHillAction(suggestion));
    suggestion->addAction(STEP_PLATEU_EXE, new ExecuteStepPlateuAction(suggestion));
    suggestion->addAction(RAMP_PLATEU_EXE, new ExecuteRampPlateuAction(suggestion));
    suggestion->addAction(SMOOTH_PLATEU_EXE, new ExecuteSmoothPlateuAction(suggestion));
    suggestion->addAction(MAXIS_PLATEU_EXE, new ExecuteMaxisPlateuAction(suggestion));
    suggestion->addAction(VOLCANO_EXE, new ExecuteVolcanoAction(suggestion));
    suggestion->addAction(ISLAND_EXE, new ExecuteIslandAction(suggestion));
    
    suggestion->setTerrain(terrain);
    
    up = new UpInterpreter(suggestion, *terrain);
    up->setViewport(1200, 800);
    up->setWindow();
    
    front = new FrontInterpreter(suggestion, *terrain);
    front->setViewport(1200, 800);
    front->setWindow();
    front->addAction(HAROLD_GEN, new GenerateHaroldAction(suggestion) );
    
    canvas = new ModelingCanvas(centralWidget);
    canvas->addObject(terrain);
    canvas->setGeometry(QRect(330, 10, 1200, 800));
    canvas->setRotation(0.0, 0.0, 0.0);
    canvas->setTranslation(0.0, 65.0, 0.0);
    terrain->changeCamera(Point(0.0, 65.0, 0.0), Point(0.0, 0.0, 0.0));


    canvas->setCamera(0.0, 80.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0, 10.0, -1.0, true);
    canvas->setViewport(0.0, 0.0, 600, 400);
    canvas->setWindow(-10, 10, -10, 10, -30, 30);
    canvas->setVision(UP);
    canvas->setUpInterpreter(up);
    canvas->setFrontInterpreter(front);
    canvas->setSuggestion(suggestion);
    
    front->setCanvas(canvas);

    upCanvas = new UpCanvas(centralWidget);
    upCanvas->setGeometry(QRect(10, 10, 300, 200));
    upCanvas->addObject(terrain);
    upCanvas->setRotation(0.0, 0.0, 0.0);
    upCanvas->setTranslation(0.0, 65.0, 0.0);
    upCanvas->setCamera(0.0, 80, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0, 10.0, -1.0, true);
    upCanvas->setViewport(0.0, 0.0, 600, 400);
    upCanvas->setWindow(-10, 10, -10, 10, -10, 10);
    upCanvas->setMainCanvas(canvas);

    frontCanvas = new FrontCanvas(centralWidget);
    frontCanvas->setGeometry(QRect(10, 225, 300, 200));
    frontCanvas->addObject(terrain);
    frontCanvas->setRotation(-80.0, 0.0, 0.0);
    frontCanvas->setTranslation(0.0, 67.0, 0.0);
    frontCanvas->setCamera(0.0, 80, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0, 10.0, -1.0, true);
    frontCanvas->setViewport(0.0, 0.0, 600, 400);
    frontCanvas->setWindow(-10, 10, -10, 10, -10, 10);
    frontCanvas->setMainCanvas(canvas);

    imageCanvas = new MyImageCanvas(centralWidget);
    imageCanvas->setGeometry(QRect(10, 440, 300, 200));
    imageCanvas->addObject(terrain);
    imageCanvas->setRotation(0.0, 0.0, 0.0);
    imageCanvas->setTranslation(0.0, 0.0, 0.0);
    imageCanvas->setCamera(0.0, 20.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,-1.0,-1.0,-1.0, false);
    imageCanvas->setViewport(0.0, 0.0, 600, 400);
    imageCanvas->setWindow(-5, 5, -5, 5, -10, 10);
    imageCanvas->setTitle("Image Canvas");

    sketchCanvas = new MySketchCanvas(centralWidget);
    sketchCanvas->setGeometry(QRect(10, 655, 300, 200));
    sketchCanvas->addObject(terrain);
    sketchCanvas->setRotation(-80.0, 0.0, 0.0);
    sketchCanvas->setTranslation(0.0, 67.0, 0.0);
    sketchCanvas->setCamera(0.0, 71.2, 0.0, 0.0, 0.0, 0.0, 0.0,0.0,-1.0,-1.0,-1.0, false);
    sketchCanvas->setViewport(0.0, 0.0, 600, 400);
    sketchCanvas->setWindow(-13, 13, -13, 13, -10, 10);
    sketchCanvas->setTitle("Sketch Canvas");

    suggestion->setModelingCanvas(canvas);
    suggestion->addAction(UPDATE_SILHOUETTE, new UpdateSilhouetteCanvasAction(suggestion, sketchCanvas));
    
    annotationWidget = new MyAnnotation(centralWidget);
    annotationWidget->setGeometry(QRect(1220, 800, 300, 30));
    annotationWidget->setTerrain(terrain);
    annotationWidget->setSuggestion(suggestion);

    up->setAnnotation(annotationWidget);
    
    connect(annotationWidget, SIGNAL(returnPressed()), this, SLOT(setTopography()));
}

void MainWindow::newFile(){
    terrain->clear();
    terrain->startBase();
    suggestion->clear();
}

void MainWindow::saveFile(){
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"), "data",
                                                     tr("type (*.sbt *.txt)"));

    this->terrain->writeData(fileName.toStdString());
}

void MainWindow::saveImage(){
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save Image"), "data",
                                                     tr("type (*.png *.jpg)"));

    QImage image = this->canvas->grabFrameBuffer();
    image.save(fileName);
}

void MainWindow::saveDemImage(){
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save Image"), "data",
                                                     tr("type (*.png *.jpg)"));

    vector< vector<float> >&dem = terrain->getDem();
    QImage image = QImage(dem.size(), dem[0].size(), QImage::Format_RGB16);
    float p = this->terrain->getPeak();
    for (int i = 0; i < dem.size(); i++){
        for (int j = 0; j < dem[0].size(); j++){
            int gray = int(dem[i][j]*255 / p);
            image.setPixel(i, j,  qRgb(gray, gray, gray));
        }
    }
    image.save(fileName);
}

void MainWindow::saveCone(){
    int v = this->canvas->getScene()->getVision();
    this->canvas->setVision(DOWN);
    QImage image = this->canvas->grabFrameBuffer();
    image.save("data/cone.png");
    this->canvas->setVision(v);
}

void MainWindow::openFile(){
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open"), "files",
                                                     tr("type (*.sbt *.txt)"));
    if (!fileName.isEmpty()){    
        this->terrain->clear();
        terrain->startBase();
        this->terrain->readData( fileName.toStdString() );
        if (this->terrain->getBase().size() > 5) setTopography();
    }
}

void MainWindow::setWireMode(){
    this->terrain->setMode(LINES);
    this->canvas->enableFocus();
}

void MainWindow::setNprMode(){
    this->terrain->setMode(POINTS);
    this->canvas->enableFocus();
}

void MainWindow::setPrMode(){
    this->terrain->setMode(POLYGONS);
    this->canvas->enableFocus();
}

void MainWindow::setContourMode(){
    this->terrain->setMode(CONTOUR);
    this->canvas->enableFocus();
}

void MainWindow::setBoxMode(){
    this->terrain->setMode(BOX);
    this->canvas->enableFocus();
}

void MainWindow::setSkeletonMode(){
    this->terrain->setMode(SKELETON);
    this->canvas->enableFocus();
}

void MainWindow::setConeMode(){
    this->terrain->setMode(CONE);
    this->canvas->enableFocus();
}

void MainWindow::setShadingMode(){
    this->terrain->setMode(SHADING);
    this->canvas->enableFocus();
}

void MainWindow::setNormalMode(){
    this->terrain->setMode(NORMAL);
    this->canvas->enableFocus();
}

void MainWindow::setTopography(){
    annotationWidget->disable();
    
    terrain->clear();
    terrain->finishBase();
    
    Landform* land = terrain->createLandform(RAMP_PLATEU);
    land->setHeight(2.0);
    
    Trace trace = terrain->getBase();
    if ( trace.size() > 3 ){
        for (int i = 0; i < trace.size(); i++){
            land->addBasePoint(trace[i]);
        }
        terrain->updateTerrain();
    }
    terrain->startBase();
    this->canvas->enableFocus();
}

void MainWindow::setShowHill(){
    if (hill->checkState() == 0 ) {
        up->showHill = false;
    }
    else{
        up->showHill = true;
    }
    this->canvas->enableFocus();
}

void MainWindow::setShowPlateu(){
    if (plateu->checkState() == 0 ) {
        up->showPlateu = false;
    }
    else{
        up->showPlateu = true;
    }
    this->canvas->enableFocus();
}

void MainWindow::setShowVolcano(){
    if (volcano->checkState() == 0 ) {
        up->showVolcano = false;
    }
    else{
        up->showVolcano = true;
    }
    this->canvas->enableFocus();
}

void MainWindow::setShowIsland(){
    if (island->checkState() == 0 ) {
        up->showIsland = false;
    }
    else{
        up->showIsland = true;
    }
    this->canvas->enableFocus();
}


void MainWindow::setLandHeight(){
    Landform* land = terrain->getLandform();
    if(land){
        terrain->landHeight = max(0.5, 5.0*heightBar->value()/(heightBar->maximum()-heightBar->minimum()));
        land->setHeight(terrain->landHeight);
        land->finishBase();
        terrain->updateTerrain();
    }
    this->canvas->enableFocus();
}

void MainWindow::setLandSlope(){
    Landform* land = terrain->getLandform();
    if(land){
        terrain->landSlope = max(0.5, 5.0*slopeBar->value()/(slopeBar->maximum()-slopeBar->minimum()));
        land->setSlopeInc(terrain->landSlope);
        land->finishBase();
        terrain->updateTerrain();
    }
    this->canvas->enableFocus();
}

void MainWindow::checkShowMesh(){
    if (showMesh->checkState() == 0){
        terrain->setShowMesh(false);
    }
    else{
        terrain->setShowMesh(true);
    }
    this->canvas->enableFocus();
}

void MainWindow::checkColorize(){
    if (colorize->checkState() == 0){
        terrain->setColorize(false);
    }
    else{
        terrain->setColorize(true);
    }
    this->canvas->enableFocus();
}


void MainWindow::checkNoise(){
    if (noise->checkState() == 0){
        terrain->setNoise(false);
    }
    else{
        terrain->setNoise(true);
    }
    this->canvas->enableFocus();
}

void MainWindow::checkBumpMapping(){
    if (bumpmapping->checkState() == 0){
        terrain->setBumpMapping(false);
    }
    else{
        terrain->setBumpMapping(true);
    }
    this->canvas->enableFocus();
}
