#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <stdio.h>
#include <string.h>

#include <QtGui>
#include <QMainWindow>
#include <QWidget>
#include <QImage>
#include <QRgb>
#include <QScrollArea>

#include "mycanvas.h"
#include "mysuggestion.h"

#include "myannotation.h"

#include "kernel.h"
#include "maxis.h"
#include "skeleton.h"





#include "detail.h"






class UpInterpreter;
class FrontInterpreter;

class MainWindow :  public QMainWindow
{
    Q_OBJECT

    private:
        QCheckBox* hill;
        QCheckBox* plateu;
        QCheckBox* volcano;
        QCheckBox* island;

        QScrollBar* heightBar;
        QScrollBar* slopeBar;

        QCheckBox* colorize;
        QCheckBox* showMesh;
        QCheckBox* noise;
        QCheckBox* bumpmapping;

        Terrain* terrain;
        
        UpInterpreter* up;
        FrontInterpreter* front;

        ModelingCanvas* canvas;
        UpCanvas* upCanvas;
        FrontCanvas* frontCanvas;
        MyImageCanvas* imageCanvas;
        MySketchCanvas* sketchCanvas;

        MySuggestion* suggestion;
        
        MyAnnotation* annotationWidget;
        
    public:
        MainWindow(QMainWindow *parent = 0);
        ~MainWindow(){};
        
    public slots:    
        void newFile();
        void saveFile();
        void saveImage();
        void saveDemImage();
        void saveCone();
        void openFile();
        
        void setWireMode();
        void setNprMode();
        void setPrMode();
        void setContourMode();
        void setBoxMode();
        void setSkeletonMode();
        void setConeMode();
        void setShadingMode();
        void setNormalMode();
        
        void setTopography();

        void setShowHill();
        void setShowPlateu();
        void setShowVolcano();
        void setShowIsland();

        void setLandHeight();
        void setLandSlope();

        void checkShowMesh();
        void checkColorize();
        void checkNoise();
        void checkBumpMapping();
};

#endif // MAINWINDOW_H
