#include "mycanvas.h"

#include "interpreter.h"
#include "myscene.h"

//////////////////////
//  ModelingCanvas  //
//////////////////////

ModelingCanvas::ModelingCanvas(QWidget* parent) : SuggestionCanvas(parent){
    this->viewWidth = 600;
    this->viewHeight = 400;

    this->windLeft = -5;
    this->windRight = 5;
    this->windBottom = -5;
    this->windTop = 5;
        
    this->angle_x = 0.0;
    this->angle_y = 0.0;
    this->angle_z = 0.0;
    
    this->interval = 50;
    this->idTimer = this->startTimer(this->interval);
    
    this->scene = new MyScene(this, NULL);

    setFocusPolicy(Qt::WheelFocus);
}

ModelingCanvas::~ModelingCanvas(){

}

void ModelingCanvas::setUpInterpreter(Interpreter* _up){
    up = _up;
}

void ModelingCanvas::setFrontInterpreter(Interpreter* _front){
    front = _front;
}

void ModelingCanvas::setVision(int v){
    scene->setVision(v);
    glDraw();
}

void ModelingCanvas::addObject(GraphicObject* object){
    this->terrain = (Terrain*) object;
    this->scene->addObject(terrain);
}

void ModelingCanvas::keyPressEvent(QKeyEvent *event){
    switch(event->key()){
            case Qt::Key_4:
                this->scene->setTranslateX( this->scene->getTranslateX() + 0.8);
                this->terrain->changeCamera(this->scene->getTranslation(), this->scene->getRotation());
                break;
            case Qt::Key_6:
                this->scene->setTranslateX( this->scene->getTranslateX() - 0.8);
                this->terrain->changeCamera(this->scene->getTranslation(), this->scene->getRotation());
                break;
            case Qt::Key_8:
                this->scene->setTranslateY( this->scene->getTranslateY() + 0.8);
                this->terrain->changeCamera(this->scene->getTranslation(), this->scene->getRotation());
                break;
            case Qt::Key_2:
                this->scene->setTranslateY( this->scene->getTranslateY() - 0.8);
                this->terrain->changeCamera(this->scene->getTranslation(), this->scene->getRotation());
                break;
            case Qt::Key_1:
                this->scene->setTranslateZ( this->scene->getTranslateZ() + 0.8);
                this->terrain->changeCamera(this->scene->getTranslation(), this->scene->getRotation());
                break;
            case Qt::Key_9:
                this->scene->setTranslateZ( this->scene->getTranslateZ() - 0.8);
                this->terrain->changeCamera(this->scene->getTranslation(), this->scene->getRotation());
                break;
            case Qt::Key_Up:
                this->scene->setRotateX(this->scene->getRotateX() - 0.8);
                this->terrain->changeCamera(this->scene->getTranslation(), this->scene->getRotation());
                this->scene->getTranslation().show(); this->scene->getRotation().show();
                break;
            case Qt::Key_Down:
                this->scene->setRotateX(this->scene->getRotateX() + 0.8);
                this->terrain->changeCamera(this->scene->getTranslation(), this->scene->getRotation());
                break;
            case Qt::Key_Left:
                this->scene->setRotateY(this->scene->getRotateY() + 0.8);
                this->terrain->changeCamera(this->scene->getTranslation(), this->scene->getRotation());
                break;
            case Qt::Key_Right:
                this->scene->setRotateY(this->scene->getRotateY() - 0.8);
                this->terrain->changeCamera(this->scene->getTranslation(), this->scene->getRotation());
                break;
            case Qt::Key_S:
                this->terrain->setMode(POLYGONS);
                break;
            case Qt::Key_W:
                this->terrain->setMode(LINES);
                break;
            case Qt::Key_P:
                this->terrain->setMode(POINTS);
                break;
            case Qt::Key_Plus:
                this->terrain->changeLevel(1);
                break;
            case Qt::Key_Minus:
                this->terrain->changeLevel(-1);
                break;
            case Qt::Key_E:
                this->terrain->startBase();
                this->terrain->createLandform(RAMP_PLATEU);
                this->suggestion->clear();
                break;
        }
}

void ModelingCanvas::mousePressEvent(QMouseEvent *event){
    glDraw();
    Point p(float(event->x()), float(event->y()));
    
    if (this->scene->getVision() == UP and this->up){
        up->startTrace();
        up->addCursor(p);
    }
    else if (this->scene->getVision() == FRONT and this->front){
        front->startTrace();
        front->addCursor(p);
    }
}

void ModelingCanvas::mouseMoveEvent(QMouseEvent *event){
    glDraw();
    if (this->scene->getVision() == UP and this->up){
        up->addCursor( Point(float(event->x()), float(event->y())) );
    }
    else if (this->scene->getVision() == FRONT and this->front){
        front->addCursor( Point(float(event->x()), float(event->y())) );
    }
}

void ModelingCanvas::mouseReleaseEvent(QMouseEvent *event){
    glDraw();
    if (this->scene->getVision() == UP and this->up){
        up->addCursor( Point(float(event->x()), float(event->y())) );
        up->finishTrace();
    }
    else if (this->scene->getVision() == FRONT and this->front){
        front->addCursor( Point(float(event->x()), float(event->y())) );
        front->finishTrace();
    }
}

void ModelingCanvas::paintGL(){
    scene->draw();
    glFlush();
}

////////////////////
//  WidgetCanvas  //
////////////////////

WidgetCanvas::WidgetCanvas(){

}

WidgetCanvas::~WidgetCanvas(){

}

void WidgetCanvas::setMainCanvas(ModelingCanvas* c){
    canvas = c;
}

void WidgetCanvas::addObject(GraphicObject* object){
    this->terrain = (Terrain*) object;
}


////////////////
//  UpCanvas  //
////////////////

UpCanvas::UpCanvas(QWidget* _par) : SceneCanvas( _par){

}

UpCanvas::~UpCanvas(){

}

void UpCanvas::mousePressEvent(QMouseEvent *event){
    canvas->setVision(UP);
}


void UpCanvas::paintGL(){
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(translation.getX(), translation.getY(), translation.getZ());
    glRotatef(angle_x, 1.0, 0.0, 0.0);
    glRotatef(angle_y, 0.0, 1.0, 0.0);
    glRotatef(angle_z, 0.0, 0.0, 1.0);
    
    terrain->draw(true, false, true, false);

    glColor3f(0.0, 0.0, 0.0);
    renderText(5, 15, QString("Up Terrain"));

    glFlush();
}


///////////////////
//  FrontCanvas  //
///////////////////

FrontCanvas::FrontCanvas(QWidget* _par) : SceneCanvas( _par){

}

FrontCanvas::~FrontCanvas(){

}

void FrontCanvas::mousePressEvent(QMouseEvent *event){
    canvas->setVision(FRONT);
}

void FrontCanvas::paintGL(){
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(translation.getX(), translation.getY(), translation.getZ());
    glRotatef(angle_x, 1.0, 0.0, 0.0);
    glRotatef(angle_y, 0.0, 1.0, 0.0);
    glRotatef(angle_z, 0.0, 0.0, 1.0);
    
    terrain->draw(false, true, false, false);

    glColor3f(0.0, 0.0, 0.0);
    renderText(5, 15, QString("Front Terrain"));

    glFlush();
}


/////////////////////
//  MyImageCanvas  //
/////////////////////

MyImageCanvas::MyImageCanvas(QWidget* _par) : ImageCanvas(_par){

}

MyImageCanvas::~MyImageCanvas(){

}

void MyImageCanvas::updateImage(){
    vector< vector<float> >&dem = terrain->getDem();
    image = QImage(dem.size(), dem[0].size(), QImage::Format_RGB16);
    float p = this->terrain->getPeak();
    for (int i = 0; i < dem.size(); i++){
        for (int j = 0; j < dem[0].size(); j++){
            int gray = int(dem[i][j]*255 / p);
            image.setPixel(i, j,  qRgb(gray, gray, gray));
        }
    }
}

//////////////////////
//  MySketchCanvas  //
//////////////////////

MySketchCanvas::MySketchCanvas(QWidget* _par) : SketchCanvas(_par){

}

MySketchCanvas::~MySketchCanvas(){

}

void MySketchCanvas::updateSketch(){
    Trace t = this->terrain->getSilhouette();

    if (t.size() > 3){
        trace.setPoints(t.getPoints());
        this->trace.rotateXZ();
        this->trace.adjust(true);
    }
    else if (this->terrain->getBase().size() > 3){
        t = this->terrain->getBase();
        trace.setPoints(t.getPoints());
        for(int i = 0; i < trace.size(); i++){
            trace[i].rotate(90.0,0.0,0.0);
        }
    }
}

//////////////////////
//  LandformCanvas  //
//////////////////////

LandformCanvas::LandformCanvas( QWidget* _par ) : SuggestionCanvas( _par) {
    this->viewWidth = 600;
    this->viewHeight = 400;

    this->windLeft = -5;
    this->windRight = 5;
    this->windBottom = -5;
    this->windTop = 5;
        
    this->angle_x = 0.0;
    this->angle_y = 0.0;
    this->angle_z = 0.0;
    
    this->interval = 50;
    this->idTimer = this->startTimer(this->interval);
}

LandformCanvas::~LandformCanvas(){

}

void LandformCanvas::addObject(GraphicObject* object){
    this->terrain = (Terrain*) object;
}

//void LandformCanvas::initializeGL(){
//    GLfloat  dimLight[] = { 0.3f, 0.3f, 0.3f, 1.0f };
//    GLfloat  sourceLight[] = { 0.95f, 0.95f, 0.95f, 10.0f };
//    GLfloat	 lightPos[] = { 0.0f, 50.0f, 0.0f, 10.0f };
//
//    glEnable(GL_DEPTH_TEST);
//
//    glEnable(GL_LIGHTING);
//
//    glLightfv(GL_LIGHT0, GL_AMBIENT, dimLight);
//    glLightfv(GL_LIGHT0,GL_DIFFUSE,sourceLight);
//    glLightfv(GL_LIGHT0,GL_POSITION,lightPos);
//    glEnable(GL_LIGHT0);
//
//    glEnable(GL_COLOR_MATERIAL);
//
//    glClearColor(0.80f, 0.80f, 0.80f, 1.0f );
//    glClearDepth(1000.0);
//}

void LandformCanvas::paintGL(){
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(translation.getX(), translation.getY(), translation.getZ());
    glRotatef(angle_x, 1.0, 0.0, 0.0);
    glRotatef(angle_y, 0.0, 1.0, 0.0);
    glRotatef(angle_z, 0.0, 0.0, 1.0);

    terrain->draw(true, true, true, true);

    glColor3f(0.0, 0.0, 0.0);
    renderText(5, 15, QString(title.data()));

    glFlush();
}
