#ifndef MY_CANVAS_H
#define MY_CANVAS_H

#include <vector>
#include <cstdio>

#include <QtSvg>

#include <QGLWidget>
#include <QPainter>
#include <QKeyEvent>

#include <QtOpenGL>
#include <QImage>
//#include <QTimeLine>
#include <QSvgRenderer>

#include <GL/gl.h>

#include "canvas.h"
#include "primitive.h"
#include "terrain.h"
#include "graphicobject.h"
#include "suggestion.h"
#include "suggestioncanvas.h"

using namespace std;

class Interpreter;

class ModelingCanvas : public SuggestionCanvas{
    protected:
        Terrain* terrain;
        
        Interpreter* up;
        Interpreter* front;
        
    public:
        ModelingCanvas(QWidget* parent);
        ~ModelingCanvas();
        
        void setVision(int v);
        
        void setUpInterpreter(Interpreter* _up);
        void setFrontInterpreter(Interpreter* _front);
        
        void addObject(GraphicObject* object);
        
        Point getScenePoint(Point cursor);
        vector<Selected> parseSelectedBuffer(GLuint selectBuff[SELECT_BUFFER_LENGTH]);
        vector<Selected> selectElements(int xPos, int yPos);
        
        
        void keyPressEvent(QKeyEvent *event);
        void mousePressEvent(QMouseEvent *event);
        void mouseMoveEvent(QMouseEvent *event);
        void mouseReleaseEvent(QMouseEvent *event);
        
        void paintGL();
};


class WidgetCanvas{
    protected:
        ModelingCanvas* canvas;
        Terrain* terrain;
        
    public:
        WidgetCanvas();
        ~WidgetCanvas();
        
        void setMainCanvas(ModelingCanvas* c);
        void addObject(GraphicObject* object);
}; 


class UpCanvas : public WidgetCanvas, public SceneCanvas{
    public:
        UpCanvas(QWidget* parent);
        ~UpCanvas();
        
        void mousePressEvent(QMouseEvent *event);
        void paintGL();
};


class FrontCanvas : public WidgetCanvas, public SceneCanvas{
    public:
        FrontCanvas(QWidget* parent);
        ~FrontCanvas();
        
        void mousePressEvent(QMouseEvent *event);
//        void initializeGL();
        void paintGL();
};


class MyImageCanvas : public WidgetCanvas, public ImageCanvas{
    public:
        MyImageCanvas(QWidget* _par);
        ~MyImageCanvas();
        
        void updateImage();
};


class MySketchCanvas : public WidgetCanvas, public SketchCanvas{
    public:
        MySketchCanvas(QWidget* _par);
        ~MySketchCanvas();

        void updateSketch();
};


class LandformCanvas : public SuggestionCanvas{
    private:
        Terrain* terrain;
        
    public:
        LandformCanvas(QWidget* _par);
        ~LandformCanvas();
        
        void addObject(GraphicObject* object);

//        void initializeGL();
        void paintGL();
};
#endif // MY_CANVAS_H
