#ifndef SUGGESTION_ACTION_H
#define SUGGESTION_ACTION_H

#include "mysuggestion.h"
#include "mycanvas.h"
#include "trace.h"

#include "maxis.h"

class GenerateHaroldAction : public Action{
    private:
        MySuggestion* suggestion;
        
    public:
        GenerateHaroldAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~GenerateHaroldAction(){}

        void act(){
            float len;
            float w = 25.0;
            float h = 25.0;

            Terrain* haroldTerrain = new Terrain();
            haroldTerrain->generatePlane(50,50);
            
            Landform* land = haroldTerrain->createLandform(HAROLD_HILL);
            haroldTerrain->addLandform(land);
            
            Trace trace = this->suggestion->getTerrain()->getSilhouette();
            if ( trace.size() > 3 ){
                trace.rotateXZ();
                Point c = trace.getCentroide(); 
                trace.translate( Point( (-1)*c.getX(), 0.0f, (-1)*c.getZ() ));
                trace.adjust(true);
                
                len = ( trace.getPoints().front() - trace.getPoints().back() ).lenght();
                w = max(float(5.0), len );
                h = max(float(5.0), w );
                
                for (int i = 0; i < trace.size(); i++){
                    land->addSilhouettePoint(trace[i]);
                }
                land->adjustSilhouette();
            }
            haroldTerrain->setDimensions(w+2.0, h);
            haroldTerrain->updateTerrain();

            LandformCanvas* har = new LandformCanvas(this->suggestion->getCanvasParent());
            this->suggestion->setupCanvas(har);
            har->setTitle("Harold");
            har->setActId(HAROLD_EXE);
            har->addObject(haroldTerrain);
            har->setRotation(-80.0, 0.0, 0.0);
            har->setTranslation(0.0, 67.0, 0.0);
            har->setCamera(0.0, 71.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0, 10.0, -1.0, true);
            har->setViewport(0.0, 0.0, 600, 400);
            har->setWindow(-13, 13, -13, 13, -10, 10);
            this->suggestion->addSuggestion(har);
        }
};


class GenerateDistanceHillAction : public Action{
    private:
        MySuggestion* suggestion;

    public:
        GenerateDistanceHillAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~GenerateDistanceHillAction(){}

        void act(){
            Terrain* rampTerrain = new Terrain(25, 25);
            rampTerrain->generatePlane(50,50);

            Landform* land = rampTerrain->createLandform(DISTANCE_HILL);
            land->setHeight(2.0);

            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
                rampTerrain->updateTerrain();
            }

            LandformCanvas* ramp = new LandformCanvas(this->suggestion->getCanvasParent());
            this->suggestion->setupCanvas(ramp);
            ramp->setTitle("Distance Hill");
            ramp->setActId(DISTANCE_HILL_EXE);
            ramp->addObject(rampTerrain);

            ramp->setRotation(-80.0, 0.0, 0.0);
            ramp->setTranslation(0.0, 67.0, 0.0);
            ramp->setCamera(0.0, 80, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0, 10.0, -1.0, true);
            ramp->setViewport(0.0, 0.0, 600, 400);
            ramp->setWindow(-10, 10, -10, 10, -10, 10);

            rampTerrain->setMode(SHADING);
            rampTerrain->setShowMesh(true);
            rampTerrain->setColorize(true);
            rampTerrain->changeCamera(Point(0.0, 65.0, 0.0),Point(-0.1,0.0,0.0));
            for (int i = 0; i < 7; i++){
                rampTerrain->changeLevel(1);
            }

            this->suggestion->addSuggestion(ramp);
        }
};

class GenerateMaxisDistanceHillAction : public Action{
    private:
        MySuggestion* suggestion;

    public:
        GenerateMaxisDistanceHillAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~GenerateMaxisDistanceHillAction(){}

        void act(){
            Terrain* rampTerrain = new Terrain(25, 25);
            rampTerrain->generatePlane(50,50);

            Landform* land = rampTerrain->createLandform(MAXISDISTANCE_HILL);
            land->setHeight(2.0);

            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
                rampTerrain->updateTerrain();
            }

            LandformCanvas* ramp = new LandformCanvas(this->suggestion->getCanvasParent());
            this->suggestion->setupCanvas(ramp);
            ramp->setTitle("Maxis Distance Hill");
            ramp->setActId(MAXISDISTANCE_HILL_EXE);
            ramp->addObject(rampTerrain);

            ramp->setRotation(-80.0, 0.0, 0.0);
            ramp->setTranslation(0.0, 67.0, 0.0);
            ramp->setCamera(0.0, 80, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0, 10.0, -1.0, true);
            ramp->setViewport(0.0, 0.0, 600, 400);
            ramp->setWindow(-10, 10, -10, 10, -10, 10);

            rampTerrain->setMode(SHADING);
            rampTerrain->setShowMesh(true);
            rampTerrain->setColorize(true);
            rampTerrain->changeCamera(Point(0.0, 65.0, 0.0),Point(-0.1,0.0,0.0));
            for (int i = 0; i < 7; i++){
                rampTerrain->changeLevel(1);
            }

            this->suggestion->addSuggestion(ramp);
        }
};

class GenerateNormalizedHillAction : public Action{
    private:
        MySuggestion* suggestion;

    public:
        GenerateNormalizedHillAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~GenerateNormalizedHillAction(){}

        void act(){
            Terrain* rampTerrain = new Terrain(25, 25);
            rampTerrain->generatePlane(50,50);

            Landform* land = rampTerrain->createLandform(NORMALIZED_HILL);
            land->setHeight(2.0);

            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
                rampTerrain->updateTerrain();
            }

            LandformCanvas* ramp = new LandformCanvas(this->suggestion->getCanvasParent());
            this->suggestion->setupCanvas(ramp);
            ramp->setTitle("Normalized Hill");
            ramp->setActId(NORMALIZED_HILL_EXE);
            ramp->addObject(rampTerrain);

            ramp->setRotation(-80.0, 0.0, 0.0);
            ramp->setTranslation(0.0, 67.0, 0.0);
            ramp->setCamera(0.0, 80, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0, 10.0, -1.0, true);
            ramp->setViewport(0.0, 0.0, 600, 400);
            ramp->setWindow(-10, 10, -10, 10, -10, 10);

            rampTerrain->setMode(SHADING);
            rampTerrain->setShowMesh(true);
            rampTerrain->setColorize(true);
            rampTerrain->changeCamera(Point(0.0, 65.0, 0.0),Point(-0.1,0.0,0.0));
            for (int i = 0; i < 7; i++){
                rampTerrain->changeLevel(1);
            }

            this->suggestion->addSuggestion(ramp);
        }
};

class GenerateStepPlateuAction : public Action{
    private:
        MySuggestion* suggestion;
        
    public:
        GenerateStepPlateuAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~GenerateStepPlateuAction(){}

        void act(){
            Terrain* rampTerrain = new Terrain(25, 25);
            rampTerrain->generatePlane(50,50);

            Landform* land = rampTerrain->createLandform(STEP_PLATEU);
            land->setHeight(2.0);

            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
                rampTerrain->updateTerrain();
            }

            LandformCanvas* ramp = new LandformCanvas(this->suggestion->getCanvasParent());
            this->suggestion->setupCanvas(ramp);
            ramp->setTitle("Step Plateu");
            ramp->setActId(STEP_PLATEU_EXE);
            ramp->addObject(rampTerrain);

            ramp->setRotation(-80.0, 0.0, 0.0);
            ramp->setTranslation(0.0, 67.0, 0.0);
            ramp->setCamera(0.0, 80, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0, 10.0, -1.0, true);
            ramp->setViewport(0.0, 0.0, 600, 400);
            ramp->setWindow(-10, 10, -10, 10, -10, 10);

            rampTerrain->setMode(SHADING);
            rampTerrain->setShowMesh(true);
            rampTerrain->setColorize(true);
            rampTerrain->changeCamera(Point(0.0, 65.0, 0.0),Point(-0.1,0.0,0.0));
            for (int i = 0; i < 7; i++){
                rampTerrain->changeLevel(1);
            }

            this->suggestion->addSuggestion(ramp);
        }
};


class GenerateRampPlateuAction : public Action{
    private:
        MySuggestion* suggestion;
        
    public:
        GenerateRampPlateuAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~GenerateRampPlateuAction(){}

        void act(){
            Terrain* rampTerrain = new Terrain(25, 25);
            rampTerrain->generatePlane(50,50);
            
            Landform* land = rampTerrain->createLandform(RAMP_PLATEU);
            land->setHeight(2.0);
            
            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
                rampTerrain->updateTerrain();
            }

            LandformCanvas* ramp = new LandformCanvas(this->suggestion->getCanvasParent());
            this->suggestion->setupCanvas(ramp);
            ramp->setTitle("Ramp Plateu");
            ramp->setActId(RAMP_PLATEU_EXE);
            ramp->addObject(rampTerrain);

            ramp->setRotation(-80.0, 0.0, 0.0);
            ramp->setTranslation(0.0, 67.0, 0.0);
            ramp->setCamera(0.0, 80, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0, 10.0, -1.0, true);
            ramp->setViewport(0.0, 0.0, 600, 400);
            ramp->setWindow(-10, 10, -10, 10, -10, 10);

            rampTerrain->setMode(SHADING);
            rampTerrain->setShowMesh(true);
            rampTerrain->setColorize(true);
            rampTerrain->changeCamera(Point(0.0, 65.0, 0.0),Point(-0.1,0.0,0.0));
            for (int i = 0; i < 7; i++){
                rampTerrain->changeLevel(1);
            }

            this->suggestion->addSuggestion(ramp);
        }
};


class GenerateSmoothPlateuAction : public Action{
    private:
        MySuggestion* suggestion;
        
    public:
        GenerateSmoothPlateuAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~GenerateSmoothPlateuAction(){}

        void act(){
            Terrain* rampTerrain = new Terrain(25, 25);
            rampTerrain->generatePlane(50,50);

            Landform* land = rampTerrain->createLandform(SMOOTH_PLATEU);
            land->setHeight(2.0);

            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
                rampTerrain->updateTerrain();
            }

            LandformCanvas* ramp = new LandformCanvas(this->suggestion->getCanvasParent());
            this->suggestion->setupCanvas(ramp);
            ramp->setTitle("Smooth Plateu");
            ramp->setActId(SMOOTH_PLATEU_EXE);
            ramp->addObject(rampTerrain);

            ramp->setRotation(-80.0, 0.0, 0.0);
            ramp->setTranslation(0.0, 67.0, 0.0);
            ramp->setCamera(0.0, 80, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0, 10.0, -1.0, true);
            ramp->setViewport(0.0, 0.0, 600, 400);
            ramp->setWindow(-10, 10, -10, 10, -10, 10);

            rampTerrain->setMode(SHADING);
            rampTerrain->setShowMesh(true);
            rampTerrain->setColorize(true);
            rampTerrain->changeCamera(Point(0.0, 65.0, 0.0),Point(-0.1,0.0,0.0));
            for (int i = 0; i < 7; i++){
                rampTerrain->changeLevel(1);
            }

            this->suggestion->addSuggestion(ramp);
        }
};

class GenerateMaxisPlateuAction : public Action{
    private:
        MySuggestion* suggestion;

    public:
        GenerateMaxisPlateuAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~GenerateMaxisPlateuAction(){}

        void act(){
            Terrain* rampTerrain = new Terrain(25, 25);
            rampTerrain->generatePlane(50,50);

            Landform* land = rampTerrain->createLandform(MAXIS_PLATEU);
            land->setHeight(2.0);

            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
                rampTerrain->updateTerrain();
            }

            LandformCanvas* ramp = new LandformCanvas(this->suggestion->getCanvasParent());
            this->suggestion->setupCanvas(ramp);
            ramp->setTitle("Maxis Plateu");
            ramp->setActId(MAXIS_PLATEU_EXE);
            ramp->addObject(rampTerrain);

            ramp->setRotation(-80.0, 0.0, 0.0);
            ramp->setTranslation(0.0, 67.0, 0.0);
            ramp->setCamera(0.0, 80, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0, 10.0, -1.0, true);
            ramp->setViewport(0.0, 0.0, 600, 400);
            ramp->setWindow(-10, 10, -10, 10, -10, 10);

            rampTerrain->setMode(SHADING);
            rampTerrain->setShowMesh(true);
            rampTerrain->setColorize(true);
            rampTerrain->changeCamera(Point(0.0, 65.0, 0.0),Point(-0.1,0.0,0.0));
            for (int i = 0; i < 7; i++){
                rampTerrain->changeLevel(1);
            }

            this->suggestion->addSuggestion(ramp);
        }
};


class GenerateVolcanoAction : public Action{
    private:
        MySuggestion* suggestion;
        
    public:
        GenerateVolcanoAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~GenerateVolcanoAction(){}

        void act(){
            Terrain* rampTerrain = new Terrain(25, 25);
            rampTerrain->generatePlane(50,50);

            Landform* land = rampTerrain->createLandform(VOLCANO);
            land->setHeight(2.0);

            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
                rampTerrain->updateTerrain();
            }

            LandformCanvas* ramp = new LandformCanvas(this->suggestion->getCanvasParent());
            this->suggestion->setupCanvas(ramp);
            ramp->setTitle("Volcano");
            ramp->setActId(VOLCANO_EXE);
            ramp->addObject(rampTerrain);

            ramp->setRotation(-80.0, 0.0, 0.0);
            ramp->setTranslation(0.0, 67.0, 0.0);
            ramp->setCamera(0.0, 80, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0, 10.0, -1.0, true);
            ramp->setViewport(0.0, 0.0, 600, 400);
            ramp->setWindow(-10, 10, -10, 10, -10, 10);

            rampTerrain->setMode(SHADING);
            rampTerrain->setShowMesh(true);
            rampTerrain->setColorize(true);
            rampTerrain->changeCamera(Point(0.0, 65.0, 0.0),Point(-0.1,0.0,0.0));
            for (int i = 0; i < 7; i++){
                rampTerrain->changeLevel(1);
            }

            this->suggestion->addSuggestion(ramp);
        }
};


class GenerateIslandAction : public Action{
    private:
        MySuggestion* suggestion;

    public:
        GenerateIslandAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~GenerateIslandAction(){}

        void act(){
            Terrain* rampTerrain = new Terrain(25, 25);
            rampTerrain->generatePlane(50,50);

            Landform* land = rampTerrain->createLandform(ISLAND);
            land->setHeight(2.0);

            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
                rampTerrain->updateTerrain();
            }

            LandformCanvas* ramp = new LandformCanvas(this->suggestion->getCanvasParent());
            this->suggestion->setupCanvas(ramp);
            ramp->setTitle("Island");
            ramp->setActId(ISLAND_EXE);
            ramp->addObject(rampTerrain);

            ramp->setRotation(-80.0, 0.0, 0.0);
            ramp->setTranslation(0.0, 67.0, 0.0);
            ramp->setCamera(0.0, 80, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.0, 10.0, -1.0, true);
            ramp->setViewport(0.0, 0.0, 600, 400);
            ramp->setWindow(-10, 10, -10, 10, -10, 10);

            rampTerrain->setMode(SHADING);
            rampTerrain->setShowMesh(true);
            rampTerrain->setColorize(true);
            rampTerrain->changeCamera(Point(0.0, 65.0, 0.0),Point(-0.1,0.0,0.0));
            for (int i = 0; i < 7; i++){
                rampTerrain->changeLevel(1);
            }

            this->suggestion->addSuggestion(ramp);
        }
};


class UpdateSilhouetteCanvasAction : public Action{
    private:
        MySuggestion* suggestion;
        MySketchCanvas* canvas;
        
    public:
        UpdateSilhouetteCanvasAction(MySuggestion* sug, MySketchCanvas* c){
            this->suggestion = sug;
            this->canvas = c;
        }
        ~UpdateSilhouetteCanvasAction(){}
        
        void act(){
            Trace trace = this->suggestion->getTerrain()->getSilhouette();
            if (trace.size() > 3){
                trace.rotateXZ();
                trace.adjust(true);
            }
        }
};


class ExecuteHaroldAction : public Action{
    private:
        MySuggestion* suggestion;
        
    public:
        ExecuteHaroldAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~ExecuteHaroldAction(){}

        void act(){
            Landform* land = this->suggestion->getTerrain()->createLandform(HAROLD_HILL);
            Trace trace = this->suggestion->getTerrain()->getSilhouette();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addSilhouettePoint(trace[i]);
                }
            }
            this->suggestion->getTerrain()->updateTerrain();
            this->suggestion->clear();
        }
};

class ExecuteDistanceHillAction : public Action{
    private:
        MySuggestion* suggestion;

    public:
        ExecuteDistanceHillAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~ExecuteDistanceHillAction(){}

        void act(){
            Landform* land = this->suggestion->getTerrain()->createLandform(DISTANCE_HILL);
//            land->setHeight(2.0);
            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
            }
            this->suggestion->getTerrain()->updateTerrain();
            this->suggestion->clear();
        }
};

class ExecuteMaxisDistanceHillAction : public Action{
    private:
        MySuggestion* suggestion;

    public:
        ExecuteMaxisDistanceHillAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~ExecuteMaxisDistanceHillAction(){}

        void act(){
            Landform* land = this->suggestion->getTerrain()->createLandform(MAXISDISTANCE_HILL);
            land->setHeight(2.0);
            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
            }
            this->suggestion->getTerrain()->updateTerrain();
            this->suggestion->clear();
        }
};

class ExecuteNormalizedHillAction : public Action{
    private:
        MySuggestion* suggestion;

    public:
        ExecuteNormalizedHillAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~ExecuteNormalizedHillAction(){}

        void act(){
            Landform* land = this->suggestion->getTerrain()->createLandform(NORMALIZED_HILL);
            land->setHeight(2.0);
            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
            }
            this->suggestion->getTerrain()->updateTerrain();
            this->suggestion->clear();
        }
};



class ExecuteStepPlateuAction : public Action{
    private:
        MySuggestion* suggestion;
        
    public:
        ExecuteStepPlateuAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~ExecuteStepPlateuAction(){}

        void act(){
            Landform* land = this->suggestion->getTerrain()->createLandform(STEP_PLATEU);
            land->setHeight(2.0);
            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
            }
            this->suggestion->getTerrain()->updateTerrain();
            this->suggestion->clear();
        }
};

class ExecuteRampPlateuAction : public Action{
    private:
        MySuggestion* suggestion;
        
    public:
        ExecuteRampPlateuAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~ExecuteRampPlateuAction(){}

        void act(){
            Landform* land = this->suggestion->getTerrain()->createLandform(RAMP_PLATEU);
            land->setHeight(2.0);
            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
            }
            this->suggestion->getTerrain()->updateTerrain();
            this->suggestion->clear();
        }
};


class ExecuteSmoothPlateuAction : public Action{
    private:
        MySuggestion* suggestion;
        Trace t;
        
    public:
        ExecuteSmoothPlateuAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~ExecuteSmoothPlateuAction(){}
        
        void act(){
            Landform* land = this->suggestion->getTerrain()->createLandform(SMOOTH_PLATEU);
            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
            }
            this->suggestion->getTerrain()->updateTerrain();
//            this->suggestion->clear();
        }
};


class ExecuteMaxisPlateuAction : public Action{
    private:
        MySuggestion* suggestion;
        Trace t;

    public:
        ExecuteMaxisPlateuAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~ExecuteMaxisPlateuAction(){}

        void act(){
            Landform* land = this->suggestion->getTerrain()->createLandform(MAXIS_PLATEU);
            land->setHeight(2.0);
            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
            }
            this->suggestion->getTerrain()->updateTerrain();
            this->suggestion->clear();
        }
};

class ExecuteVolcanoAction : public Action{
    private:
        MySuggestion* suggestion;
        Trace t;

    public:
        ExecuteVolcanoAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~ExecuteVolcanoAction(){}

        void act(){
            Landform* land = this->suggestion->getTerrain()->createLandform(VOLCANO);
            land->setHeight(2.0);
            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
            }
            this->suggestion->getTerrain()->updateTerrain();
            this->suggestion->clear();
        }
};

class ExecuteIslandAction : public Action{
    private:
        MySuggestion* suggestion;
        Trace t;

    public:
        ExecuteIslandAction(MySuggestion* sug){
            this->suggestion = sug;
        }
        ~ExecuteIslandAction(){}

        void act(){
            Landform* land = this->suggestion->getTerrain()->createLandform(ISLAND);
            land->setHeight(2.0);
            Trace trace = this->suggestion->getTerrain()->getBase();
            if ( trace.size() > 3 ){
                for (int i = 0; i < trace.size(); i++){
                    land->addBasePoint(trace[i]);
                }
            }
            this->suggestion->getTerrain()->updateTerrain();
            this->suggestion->clear();
        }
};

#endif /* SUGGESTION_ACTION_H */
